@@ -17,12 +17,14 @@
 #define MOUNTPOINTLEN   1024    /* max path length for symlink; same as AFSPATHMAX */
 
 typedef struct cm_fid {
-	unsigned long cell;
-        unsigned long volume;
-        unsigned long vnode;
-        unsigned long unique;
+    afs_uint32 cell;
+    afs_uint32 volume;
+    afs_uint32 vnode;
+    afs_uint32 unique;
+    afs_uint32 hash;
 } cm_fid_t;
 
+
 /* Key used for byte range locking.  Each unique key identifies a
    unique client per cm_scache_t for the purpose of locking. */
 typedef afs_uint64 cm_key_t;
@@ -40,7 +42,7 @@ typedef struct cm_file_lock {
     osi_queue_t q;              /* list of all locks [protected by
                                    cm_scacheLock] */
     osi_queue_t fileq;		/* per-file list of locks [protected
-                                   by scp->mx]*/
+                                   by scp->rw]*/
     
     cm_user_t *userp;           /* The user to which this lock belongs
                                    to [immutable; held] */
@@ -91,15 +93,15 @@ typedef struct cm_scache {
     struct cm_scache *nextp;		/* next in hash; cm_scacheLock */
     struct cm_scache *allNextp;         /* next in all scache list; cm_scacheLock */
     cm_fid_t fid;
-    afs_uint32 flags;			/* flags; locked by mx */
+    afs_uint32 flags;			/* flags; locked by rw */
 
     /* synchronization stuff */
-    osi_mutex_t mx;			/* mutex for this structure */
+    osi_rwlock_t rw;			/* rwlock for this structure */
     osi_rwlock_t bufCreateLock;		/* read-locked during buffer creation;
                                          * write-locked to prevent buffers from
 					 * being created during a truncate op, etc.
 					 */
-    afs_uint32 refCount;		/* reference count; cm_scacheLock */
+    afs_int32 refCount;		        /* reference count; cm_scacheLock */
     osi_queueData_t *bufReadsp;		/* queue of buffers being read */
     osi_queueData_t *bufWritesp;	/* queue of buffers being written */
 
@@ -121,6 +123,7 @@ typedef struct cm_scache {
     afs_uint32 unixModeBits;		/* unix protection mode bits */
     afs_uint32 linkCount;		/* link count */
     afs_uint64 dataVersion;		/* data version */
+    afs_uint64 bufDataVersionLow;       /* range of valid cm_buf_t dataVersions */
     afs_uint32 owner; 			/* file owner */
     afs_uint32 group;			/* file owning group */
     cm_user_t *creator;			/* user, if new file */
@@ -154,7 +157,7 @@ typedef struct cm_scache {
     afs_int32    serverLock;    /* current lock we have acquired on
                                  * this file.  One of (-1), LockRead
                                  * or LockWrite. [protected by
-                                 * scp->mx].  In the future, this
+                                 * scp->rw].  In the future, this
                                  * should be replaced by a queue of
                                  * cm_server_lock_t objects which keep
                                  * track of lock type, the user for
@@ -187,9 +190,6 @@ typedef struct cm_scache {
                                    have CM_FILELOCK_FLAG_CLIENTONLY
                                    set. */
 	
-    /* volume info */
-    struct cm_volume *volp;		/* volume info; held reference */
-
     /* bulk stat progress */
     osi_hyper_t bulkStatProgress;	/* track bulk stats of large dirs */
 
@@ -345,7 +345,9 @@ extern long cm_GetSCache(cm_fid_t *, cm_scache_t **, struct cm_user *,
 
 extern cm_scache_t *cm_GetNewSCache(void);
 
-extern int cm_FidCmp(cm_fid_t *, cm_fid_t *);
+extern __inline int cm_FidCmp(cm_fid_t *, cm_fid_t *);
+
+extern void cm_SetFid(cm_fid_t *, afs_uint32 cell, afs_uint32 volume, afs_uint32 vnode, afs_uint32 unique);
 
 extern long cm_SyncOp(cm_scache_t *, struct cm_buf *, struct cm_user *,
 	struct cm_req *, afs_uint32, afs_uint32);
