@@ -148,7 +148,7 @@ BOOL InitApp (LPSTR pszCmdLineA)
             fUninstall = TRUE;
             break;
 
-		 case ':':
+         case ':':
              CopyAnsiToString(g.SmbName,pszCmdLineA);
 			 MapShareName(pszCmdLineA);
 			 break;
@@ -175,10 +175,11 @@ BOOL InitApp (LPSTR pszCmdLineA)
    else if (fUninstall)
       Shortcut_FixStartup (cszSHORTCUT_NAME, g.fStartup = FALSE);
 
-   if (fInstall)
+   if (fInstall || fUninstall)
       {
       HKEY hk;
-      if (RegCreateKey (HKEY_CURRENT_USER, AFSREG_USER_OPENAFS_SUBKEY, &hk) == 0)
+      if (RegCreateKeyEx (HKEY_CURRENT_USER, AFSREG_USER_OPENAFS_SUBKEY, 0, NULL, 0,
+                         (IsWow64()?KEY_WOW64_64KEY:0)|KEY_WRITE, NULL, &hk, NULL) == 0)
          {
          DWORD dwSize = sizeof(g.fStartup);
          DWORD dwType = REG_DWORD;
@@ -213,14 +214,16 @@ BOOL InitApp (LPSTR pszCmdLineA)
       return FALSE;
 
    HKEY hk;
-    if (RegOpenKey (HKEY_CURRENT_USER, AFSREG_USER_OPENAFS_SUBKEY, &hk) == 0)
+    if (RegOpenKeyEx (HKEY_CURRENT_USER, AFSREG_USER_OPENAFS_SUBKEY, 0,
+                       (IsWow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &hk) == 0)
     {
         DWORD dwSize = sizeof(g.fStartup);
         DWORD dwType = REG_DWORD;
         RegQueryValueEx (hk, TEXT("ShowTrayIcon"), NULL, &dwType, (PBYTE)&g.fStartup, &dwSize);
         RegCloseKey (hk);
     }
-    else if (RegOpenKey (HKEY_LOCAL_MACHINE, AFSREG_CLT_OPENAFS_SUBKEY, &hk) == 0)
+    else if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, AFSREG_CLT_OPENAFS_SUBKEY, 0,
+                            (IsWow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &hk) == 0)
       {
       DWORD dwSize = sizeof(g.fStartup);
       DWORD dwType = REG_DWORD;
@@ -399,7 +402,8 @@ BOOL IsServerInstalled (void)
    BOOL fInstalled = FALSE;
    HKEY hk;
 
-   if (RegOpenKey (HKEY_LOCAL_MACHINE, AFSREG_SVR_SVC_SUBKEY, &hk) == 0)
+   if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, AFSREG_SVR_SVC_SUBKEY, 0,
+                      (IsWow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &hk) == 0)
       {
       fInstalled = TRUE;
       RegCloseKey (hk);
