@@ -18,7 +18,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.72.2.3 2006/07/31 21:27:40 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.72.2.6.2.1 2008/05/23 14:25:35 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -57,7 +57,7 @@ int afs_fakestat_enable = 0;	/* 1: fakestat-all, 2: fakestat-crosscell */
 static int
 EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
               struct volume **avolpp, register struct vrequest *areq,
-	      afs_uint32 *acellidxp, afs_uint32 *avolnump)
+	      afs_uint32 *acellidxp, afs_uint32 *avolnump, afs_uint32 *avnoidp)
 {
     struct volume *tvp = 0;
     struct VenusFid tfid;
@@ -67,22 +67,34 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
     afs_int32 prefetch;		/* 1=>None  2=>RO  3=>BK */
     afs_int32 mtptCell, assocCell = 0, hac = 0;
     afs_int32 samecell, roname, len;
-    afs_uint32 volid, cellidx;
+    afs_uint32 volid, cellidx, vnoid = 0;
 
     cpos = afs_strchr(data, ':');	/* if cell name present */
     if (cpos) {
+	cellnum = 0;
 	volnamep = cpos + 1;
 	*cpos = 0;
-	tcell = afs_GetCellByName(data, READ_LOCK);
+	for (x = data; *x >= '0' && *x <= '9'; x++)
+	    cellnum = (cellnum * 10) + (*x - '0');
+	if (cellnum && !*x)
+	    tcell = afs_GetCell(cellnum, READ_LOCK);
+	else {
+	    tcell = afs_GetCellByName(data, READ_LOCK);
+	               cellnum = 0;
+	}
 	*cpos = ':';
     } else if (cellnum) {
 	volnamep = data;
 	tcell = afs_GetCell(cellnum, READ_LOCK);
     } else {
+	/*printf("No cellname %s , or cellnum %d , returning ENODEV\n", 
+	       data, cellnum);*/
 	return ENODEV;
     }
-    if (!tcell)
+    if (!tcell) {
+	/*printf("Lookup failed, returning ENODEV\n");*/
 	return ENODEV;
+    }
 
     cellidx = tcell->cellIndex;
     mtptCell = tcell->cellNum;	/* The cell for the mountpoint */
@@ -92,10 +104,22 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
     }
     afs_PutCell(tcell, READ_LOCK);
 
+    cpos = afs_strrchr(volnamep, ':'); /* if vno present */
+    if (cpos) 
+	*cpos = 0;
     /* Look for an all-numeric volume ID */
     volid = 0;
     for (x = volnamep; *x >= '0' && *x <= '9'; x++)
 	volid = (volid * 10) + (*x - '0');
+    if (cpos) {
+	*cpos = ':';
+	vnoid = 0;
+	if (!*x) /* allow vno with numeric volid only */
+	    for (x = (cpos + 1); *x >= '0' && *x <= '9'; x++)
+		vnoid = (vnoid * 10) + (*x - '0');
+	if (*x)
+	    vnoid = 0;
+    }
 
     /*
      * If the volume ID was all-numeric, and they didn't ask for a
@@ -108,6 +132,8 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
 	    *acellidxp = cellidx;
 	if (avolnump)
 	    *avolnump = volid;
+	if (avnoidp)
+	    *avnoidp = vnoid;
 	return 0;
     }
 
@@ -121,8 +147,10 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
 	tfid.Fid.Volume = volid;	/* remember BK volume */
 	tfid.Cell = mtptCell;
 	tvp = afs_GetVolume(&tfid, areq, WRITE_LOCK);	/* get the new one */
-	if (!tvp)
+	if (!tvp) {
+	    /*printf("afs_GetVolume failed - returning ENODEV");*/
 	    return ENODEV;	/* oops, can't do it */
+	}
 	goto done;
     }
 
@@ -157,6 +185,7 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
      * ".backup" in it, this will get the volume struct for the RW volume.
      * The RO volume will be prefetched if requested (but not returned).
      */
+    /*printf("Calling GetVolumeByName\n");*/
     tvp = afs_GetVolumeByName(volnamep, mtptCell, prefetch, areq, WRITE_LOCK);
 
     /* If no volume was found in this cell, try the associated linked cell */
@@ -185,8 +214,10 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
 	osi_FreeSmallSpace(buf);
     }
 
-    if (!tvp)
+    if (!tvp) {
+	/*printf("Couldn't find the volume\n");*/
 	return ENODEV;		/* Couldn't find the volume */
+    }
 
     /* Don't cross mountpoint from a BK to a BK volume */
     if ((states & CBackup) && (tvp->states & VBackup)) {
@@ -221,6 +252,8 @@ done:
 	*acellidxp = cellidx;
     if (avolnump)
 	*avolnump = tvp->volume;
+    if (avnoidp)
+	*avnoidp = vnoid;
     if (avolpp)
 	*avolpp = tvp;
     else
@@ -233,6 +266,7 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
 	       struct volume **avolpp, register struct vrequest *areq)
 {
     afs_int32 code;
+    afs_uint32 avnoid;
 
     AFS_STATCNT(EvalMountPoint);
 #ifdef notdef
@@ -246,15 +280,19 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
 
     /* Determine which cell and volume the mointpoint goes to */
     code = EvalMountData(avc->linkData[0], avc->linkData + 1,
-                         avc->states, avc->fid.Cell, avolpp, areq, 0, 0);
+                         avc->states, avc->fid.Cell, avolpp, areq, 0, 0,
+			 &avnoid);
     if (code) return code;
 
+    if (!avnoid)
+	avnoid = 1;
+
     if (avc->mvid == 0)
 	avc->mvid =
 	    (struct VenusFid *)osi_AllocSmallSpace(sizeof(struct VenusFid));
     avc->mvid->Cell = (*avolpp)->cell;
     avc->mvid->Fid.Volume = (*avolpp)->volume;
-    avc->mvid->Fid.Vnode = 1;
+    avc->mvid->Fid.Vnode = avnoid;
     avc->mvid->Fid.Unique = 1;
     avc->states |= CMValid;
 
@@ -1179,6 +1217,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     register afs_int32 code;
     register afs_int32 bulkcode = 0;
     int pass = 0, hit = 0;
+    int force_eval = afs_fakestat_enable ? 0 : 1;
     long dirCookie;
     extern afs_int32 afs_mariner;	/*Writing activity to log? */
     afs_hyper_t versionNo;
@@ -1192,6 +1231,10 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     AFS_STATCNT(afs_lookup);
     afs_InitFakeStat(&fakestate);
 
+    AFS_DISCON_LOCK();
+
+    /*printf("Looking up %s\n", aname);*/
+    
     if ((code = afs_InitReq(&treq, acred)))
 	goto done;
 
@@ -1199,6 +1242,11 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     ndp->ni_dvp = AFSTOV(adp);
 #endif /* AFS_OSF_ENV */
 
+    if (afs_fakestat_enable && adp->mvstat == 1) {
+       if (strcmp(aname, ".directory") == 0)
+           tryEvalOnly = 1;
+    }
+
 #if defined(AFS_DARWIN_ENV)
     /* Workaround for MacOSX Finder, which tries to look for
      * .DS_Store and Contents under every directory.
@@ -1215,6 +1263,9 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	code = afs_TryEvalFakeStat(&adp, &fakestate, &treq);
     else
 	code = afs_EvalFakeStat(&adp, &fakestate, &treq);
+
+    /*printf("Code is %d\n", code);*/
+    
     if (tryEvalOnly && adp->mvstat == 1)
 	code = ENOENT;
     if (code)
@@ -1256,6 +1307,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	    goto done;
 	}
 	/* otherwise we have the fid here, so we use it */
+	/*printf("Getting vcache\n");*/
 	tvc = afs_GetVCache(adp->mvid, &treq, NULL, NULL);
 	afs_Trace3(afs_iclSetp, CM_TRACE_GETVCDOTDOT, ICL_TYPE_FID, adp->mvid,
 		   ICL_TYPE_POINTER, tvc, ICL_TYPE_INT32, code);
@@ -1334,9 +1386,9 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
      */
     if (afs_IsDynrootMount(adp)) {
 	struct VenusFid tfid;
-	afs_uint32 cellidx, volid;
+	afs_uint32 cellidx, volid, vnoid;
 
-	code = EvalMountData('%', aname, 0, 0, NULL, &treq, &cellidx, &volid);
+	code = EvalMountData('%', aname, 0, 0, NULL, &treq, &cellidx, &volid, &vnoid);
 	if (code)
 	    goto done;
 	afs_GetDynrootMountFid(&tfid);
@@ -1515,7 +1567,8 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	/* prefetch some entries, if the dir is currently open.  The variable
 	 * dirCookie tells us where to start prefetching from.
 	 */
-	if (AFSDOBULK && adp->opens > 0 && !(adp->states & CForeign)
+	if (!AFS_IS_DISCONNECTED && 
+	    AFSDOBULK && adp->opens > 0 && !(adp->states & CForeign)
 	    && !afs_IsDynroot(adp) && !afs_InReadDir(adp)) {
 	    afs_int32 retry;
 	    /* if the entry is not in the cache, or is in the cache,
@@ -1562,8 +1615,6 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     }				/* sub-block just to reduce stack usage */
 
     if (tvc) {
-	int force_eval = afs_fakestat_enable ? 0 : 1;
-
 	if (adp->states & CForeign)
 	    tvc->states |= CForeign;
 	tvc->parentVnode = adp->fid.Fid.Vnode;
@@ -1584,6 +1635,9 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 		force_eval = 1;
 	    ReleaseReadLock(&tvc->lock);
 	}
+	if (tvc->mvstat == 1 && (tvc->states & CMValid) && tvc->mvid != NULL)
+	  force_eval = 1; /* This is now almost for free, get it correct */
+
 #if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
 	if (!(flags & AFS_LOOKUP_NOEVAL))
 	    /* don't eval mount points */
@@ -1661,25 +1715,30 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	 * be located (a Multics "connection failure").  If the volume is
 	 * read-only, we try flushing this entry from the cache and trying
 	 * again. */
-	if (pass == 0) {
-	    struct volume *tv;
-	    tv = afs_GetVolume(&adp->fid, &treq, READ_LOCK);
-	    if (tv) {
-		if (tv->states & VRO) {
-		    pass = 1;	/* try this *once* */
-		    ObtainWriteLock(&afs_xcbhash, 495);
-		    afs_DequeueCallback(adp);
-		    /* re-stat to get later version */
-		    adp->states &= ~CStatd;
-		    ReleaseWriteLock(&afs_xcbhash);
-		    osi_dnlc_purgedp(adp);
+	if (!AFS_IS_DISCONNECTED) {
+	    if (pass == 0) {
+	        struct volume *tv;
+	        tv = afs_GetVolume(&adp->fid, &treq, READ_LOCK);
+	        if (tv) {
+		    if (tv->states & VRO) {
+		        pass = 1;	/* try this *once* */
+		        ObtainWriteLock(&afs_xcbhash, 495);
+		        afs_DequeueCallback(adp);
+		        /* re-stat to get later version */
+		        adp->states &= ~CStatd;
+		        ReleaseWriteLock(&afs_xcbhash);
+		        osi_dnlc_purgedp(adp);
+		        afs_PutVolume(tv, READ_LOCK);
+		        goto redo;
+		    }
 		    afs_PutVolume(tv, READ_LOCK);
-		    goto redo;
-		}
-		afs_PutVolume(tv, READ_LOCK);
+	        }
 	    }
+	    code = ENOENT;
+	} else {
+	    /*printf("Network down in afs_lookup\n");*/
+	    code = ENETDOWN;
 	}
-	code = ENOENT;
     }
 
   done:
@@ -1709,7 +1768,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	     * rather than the vc of the mount point itself.  we can still find the
 	     * mount point's vc in the vcache by its fid. */
 #endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
-	    if (!hit) {
+	    if (!hit && force_eval) {
 		osi_dnlc_enter(adp, aname, tvc, &versionNo);
 	    } else {
 #ifdef AFS_LINUX20_ENV
@@ -1717,6 +1776,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 		code = afs_VerifyVCache(tvc, &treq);
 #else
 		afs_PutFakeStat(&fakestate);
+		AFS_DISCON_UNLOCK();
 		return 0;	/* can't have been any errors if hit and !code */
 #endif
 	    }
@@ -1733,5 +1793,6 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     }
 
     afs_PutFakeStat(&fakestate);
+    AFS_DISCON_UNLOCK();
     return code;
 }
