@@ -19,7 +19,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_read.c,v 1.26.2.6 2008/04/27 03:54:27 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_read.c,v 1.34.2.2.2.2 2008/05/23 14:25:35 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -388,12 +388,16 @@ afs_MemRead(register struct vcache *avc, struct uio *auio,
      */
     if (tdc) {
 	ReleaseReadLock(&tdc->lock);
-#if !defined(AFS_VM_RDWR_ENV)
 	/* try to queue prefetch, if needed */
-	if (!noLock) {
+	if (!noLock &&
+#ifndef AFS_VM_RDWR_ENV
+	    afs_preCache
+#else
+	    1
+#endif
+	    ) {
 	    afs_PrefetchChunk(avc, tdc, acred, &treq);
 	}
-#endif
 	afs_PutDCache(tdc);
     }
     if (!noLock)
@@ -439,6 +443,15 @@ afs_PrefetchChunk(struct vcache *avc, struct dcache *adc,
 	ReleaseReadLock(&adc->lock);
 
 	tdc = afs_GetDCache(avc, offset, areq, &j1, &j2, 2);	/* type 2 never returns 0 */
+#ifdef AFS_DISCON_ENV
+        /*
+         * In disconnected mode, type 2 can return 0 because it doesn't
+         * make any sense to allocate a dcache we can never fill
+         */
+         if (tdc == NULL)
+             return;
+#endif /* AFS_DISCON_ENV */
+
 	ObtainSharedLock(&tdc->mflock, 651);
 	if (!(tdc->mflags & DFFetchReq)) {
 	    /* ask the daemon to do the work */
@@ -508,6 +521,8 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
     if (avc && avc->vc_error)
 	return EIO;
 
+    AFS_DISCON_LOCK();
+    
     /* check that we have the latest status info in the vnode cache */
     if ((code = afs_InitReq(&treq, acred)))
 	return code;
@@ -518,6 +533,7 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 	    code = afs_VerifyVCache(avc, &treq);
 	    if (code) {
 		code = afs_CheckCode(code, &treq, 11);	/* failed to get it */
+		AFS_DISCON_UNLOCK();
 		return code;
 	    }
 	}
@@ -527,6 +543,7 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 	if (!afs_AccessOK
 	    (avc, PRSFS_READ, &treq,
 	     CHECK_MODE_BITS | CMB_ALLOW_EXEC_AS_READ)) {
+	    AFS_DISCON_UNLOCK();
 	    return afs_CheckCode(EACCES, &treq, 12);
 	}
     }
@@ -617,6 +634,14 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 		afs_PutDCache(tdc);	/* before reusing tdc */
 	    }
 	    tdc = afs_GetDCache(avc, filePos, &treq, &offset, &len, 2);
+#ifdef AFS_DISCON_ENV
+	    if (!tdc) {
+		/*printf("Network down in afs_read");*/
+	        error = ENETDOWN;
+	        break;
+	    }
+#endif /* AFS_DISCON_ENV */
+
 	    ObtainReadLock(&tdc->lock);
 	    /* now, first try to start transfer, if we'll need the data.  If
 	     * data already coming, we don't need to do this, obviously.  Type
@@ -948,6 +973,7 @@ afs_UFSRead(register struct vcache *avc, struct uio *auio,
 #else
     osi_FreeSmallSpace(tvec);
 #endif
+    AFS_DISCON_UNLOCK();
     error = afs_CheckCode(error, &treq, 13);
     return error;
 }
