@@ -23,7 +23,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_readdir.c,v 1.24.2.6 2005/10/03 02:46:32 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_readdir.c,v 1.33.18.1 2008/05/23 14:25:35 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -143,7 +143,12 @@ struct irix5_min_dirent {	/* miniature dirent structure */
 #else
 struct min_direct {		/* miniature direct structure */
     /* If struct direct changes, this must too */
-#if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if defined(AFS_DARWIN80_ENV)
+    ino_t d_fileno;
+    u_short d_reclen;
+    u_char d_type;
+    u_char d_namlen;
+#elif defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
     afs_uint32 d_fileno;
     u_short d_reclen;
     u_char d_type;
@@ -242,7 +247,10 @@ afs_readdir_type(avc, ade)
     tfid.Fid.Unique = ntohl(ade->fid.vunique);
     if ((avc->states & CForeign) == 0 && (ntohl(ade->fid.vnode) & 1)) {
 	return DT_DIR;
-    } else if ((tvc = afs_FindVCache(&tfid, 0, 0))) {
+    }
+    ObtainReadLock(&afs_xvcache);
+    if ((tvc = afs_FindVCache(&tfid, 0, 0))) {
+        ReleaseReadLock(&afs_xvcache);
 	if (tvc->mvstat) {
 	    afs_PutVCache(tvc);
 	    return DT_DIR;
@@ -261,7 +269,8 @@ afs_readdir_type(avc, ade)
 	    /* what other types does AFS support? */
 	} else
 	    afs_PutVCache(tvc);
-    }
+    } else
+        ReleaseReadLock(&afs_xvcache);
     return DT_UNKNOWN;
 }
 #endif
@@ -625,9 +634,11 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
     if (eofp)
 	*eofp = 0;
 #endif
+#ifndef AFS_64BIT_CLIENT
     if (AfsLargeFileUio(auio)	/* file is large than 2 GB */
-	||AfsLargeFileSize(auio->uio_offset, auio->uio_resid))
+	||AfsLargeFileSize(AFS_UIO_OFFSET(auio), AFS_UIO_RESID(auio)))
 	return EFBIG;
+#endif
 
     if ((code = afs_InitReq(&treq, acred))) {
 #ifdef	AFS_HPUX_ENV
@@ -637,6 +648,9 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
     }
     /* update the cache entry */
     afs_InitFakeStat(&fakestate);
+
+    AFS_DISCON_LOCK();
+
     code = afs_EvalFakeStat(&avc, &fakestate, &treq);
     if (code)
 	goto done;
@@ -710,7 +724,7 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
     auio->uio_fpflags = 0;
 #endif
     while (code == 0) {
-	origOffset = auio->afsio_offset;
+	origOffset = AFS_UIO_OFFSET(auio);
 	/* scan for the next interesting entry scan for in-use blob otherwise up point at
 	 * this blob note that ode, if non-zero, also represents a held dir page */
 	if (!(us = BlobScan(tdc, (origOffset >> 5)))
@@ -722,7 +736,7 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 		sdirEntry->d_fileno =
 		    (avc->fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
 		FIXUPSTUPIDINODE(sdirEntry->d_fileno);
-		sdirEntry->d_reclen = rlen = auio->afsio_resid;
+		sdirEntry->d_reclen = rlen = AFS_UIO_RESID(auio);
 		sdirEntry->d_namlen = o_slen;
 #if defined(AFS_SUN5_ENV) || defined(AFS_AIX32_ENV) || defined(AFS_HPUX100_ENV)
 		sdirEntry->d_off = origOffset;
@@ -752,11 +766,11 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 #if defined(AFS_SUN5_ENV)
 					len, origOffset);
 #else
-					auio->afsio_resid, origOffset);
+					AFS_UIO_RESID(auio), origOffset);
 #endif
 #endif /* AFS_HPUX_ENV */
 #if !defined(AFS_SUN5_ENV)
-		auio->afsio_resid = 0;
+		AFS_UIO_SETRESID(auio, 0);
 #endif
 	    } else {
 		/* nothin to hand over */
@@ -780,9 +794,9 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 #ifdef	AFS_SGI53_ENV
 	dirsiz =
 	    use64BitDirent ? DIRENTSIZE(n_slen) : IRIX5_DIRENTSIZE(n_slen);
-	if (dirsiz >= (auio->afsio_resid - len)) {
+	if (dirsiz >= (AFS_UIO_RESID(auio) - len)) {
 #else
-	if (DIRSIZ_LEN(n_slen) >= (auio->afsio_resid - len)) {
+	if (DIRSIZ_LEN(n_slen) >= (AFS_UIO_RESID(auio) - len)) {
 #endif /* AFS_SGI53_ENV */
 	    /* No can do no more now; ya know... at this time */
 	    DRelease((struct buffer *)nde, 0);	/* can't use this one. */
@@ -791,7 +805,7 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 		sdirEntry->d_fileno =
 		    (avc->fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
 		FIXUPSTUPIDINODE(sdirEntry->d_fileno);
-		sdirEntry->d_reclen = rlen = auio->afsio_resid;
+		sdirEntry->d_reclen = rlen = AFS_UIO_RESID(auio);
 		sdirEntry->d_namlen = o_slen;
 #if defined(AFS_SUN5_ENV) || defined(AFS_AIX32_ENV) || defined(AFS_HPUX100_ENV)
 		sdirEntry->d_off = origOffset;
@@ -819,12 +833,12 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 #else /* AFS_HPUX_ENV */
 		code =
 		    afs_readdir_move(ode, avc, auio, o_slen,
-				     auio->afsio_resid, origOffset);
+				     AFS_UIO_RESID(auio), origOffset);
 #endif /* AFS_HPUX_ENV */
 		/* this next line used to be AFSVFS40 or AIX 3.1, but is
 		 * really generic */
-		auio->afsio_offset = origOffset;
-		auio->afsio_resid = 0;
+		AFS_UIO_SETOFFSET(auio, origOffset);
+		AFS_UIO_SETRESID(auio, 0);
 	    } else {		/* trouble, can't give anything to the user! */
 		/* even though he has given us a buffer, 
 		 * even though we have something to give us,
@@ -885,8 +899,7 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 	if (ode)
 	    DRelease((struct buffer *)ode, 0);
 	ode = nde;
-	auio->afsio_offset =
-	    (afs_int32) ((us + afs_dir_NameBlobs(nde->name)) << 5);
+	AFS_UIO_SETOFFSET(auio, (afs_int32) ((us + afs_dir_NameBlobs(nde->name)) << 5));
     }
     if (ode)
 	DRelease((struct buffer *)ode, 0);
@@ -900,6 +913,7 @@ afs_readdir(OSI_VC_ARG(avc), auio, acred)
 #ifdef	AFS_HPUX_ENV
     osi_FreeSmallSpace((char *)sdirEntry);
 #endif
+    AFS_DISCON_UNLOCK();
     afs_PutFakeStat(&fakestate);
     code = afs_CheckCode(code, &treq, 28);
     return code;
@@ -947,11 +961,13 @@ afs1_readdir(avc, auio, acred)
 	return code;
     }
     afs_InitFakeStat(&fakestate);
+    AFS_DISCON_LOCK();
     code = afs_EvalFakeStat(&avc, &fakestate, &treq);
     if (code) {
 #ifdef	AFS_HPUX_ENV
 	osi_FreeSmallSpace((char *)sdirEntry);
 #endif
+	AFS_DISCON_UNLOCK();
 	afs_PutFakeStat(&fakestate);
 	return code;
     }
@@ -1000,7 +1016,7 @@ afs1_readdir(avc, auio, acred)
     auio->uio_fpflags = 0;
 #endif
     while (code == 0) {
-	origOffset = auio->afsio_offset;
+	origOffset = AFS_UIO_OFFSET(auio);
 
 	/* scan for the next interesting entry scan for in-use blob otherwise up point at
 	 * this blob note that ode, if non-zero, also represents a held dir page */
@@ -1013,7 +1029,7 @@ afs1_readdir(avc, auio, acred)
 		sdirEntry->d_fileno =
 		    (avc->fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
 		FIXUPSTUPIDINODE(sdirEntry->d_fileno);
-		sdirEntry->d_reclen = rlen = auio->afsio_resid;
+		sdirEntry->d_reclen = rlen = AFS_UIO_RESID(auio);
 		sdirEntry->d_namlen = o_slen;
 		sdirEntry->d_off = origOffset;
 		AFS_UIOMOVE((char *)sdirEntry, sizeof(*sdirEntry), UIO_READ,
@@ -1040,9 +1056,9 @@ afs1_readdir(avc, auio, acred)
 #else
 		code =
 		    afs_readdir_move(ode, avc, auio, o_slen,
-				     auio->afsio_resid, origOffset);
+				     AFS_UIO_RESID(auio), origOffset);
 #endif /* AFS_HPUX_ENV */
-		auio->afsio_resid = 0;
+		AFS_UIO_SETRESID(auio, 0);
 	    } else {
 		/* nothin to hand over */
 	    }
@@ -1062,7 +1078,7 @@ afs1_readdir(avc, auio, acred)
 #else
 	n_slen = strlen(nde->name);
 #endif
-	if (NDIRSIZ_LEN(n_slen) >= (auio->afsio_resid - len)) {
+	if (NDIRSIZ_LEN(n_slen) >= (AFS_UIO_RESID(auio) - len)) {
 	    /* No can do no more now; ya know... at this time */
 	    DRelease(nde, 0);	/* can't use this one. */
 	    if (len) {
@@ -1070,7 +1086,7 @@ afs1_readdir(avc, auio, acred)
 		sdirEntry->d_fileno =
 		    (avc->fid.Fid.Volume << 16) + ntohl(ode->fid.vnode);
 		FIXUPSTUPIDINODE(sdirEntry->d_fileno);
-		sdirEntry->d_reclen = rlen = auio->afsio_resid;
+		sdirEntry->d_reclen = rlen = AFS_UIO_RESID(auio);
 		sdirEntry->d_namlen = o_slen;
 		sdirEntry->d_off = origOffset;
 		AFS_UIOMOVE((char *)sdirEntry, sizeof(*sdirEntry), UIO_READ,
@@ -1096,11 +1112,11 @@ afs1_readdir(avc, auio, acred)
 #else
 		code =
 		    afs_readdir_move(ode, avc, auio, o_slen,
-				     auio->afsio_resid, origOffset);
+				     AFS_UIO_RESID(auio), origOffset);
 #endif /* AFS_HPUX_ENV */
 		/* this next line used to be AFSVFS40 or AIX 3.1, but is really generic */
-		auio->afsio_offset = origOffset;
-		auio->afsio_resid = 0;
+		AFS_UIO_SETOFFSET(auio, origOffset);
+		AFS_UIO_SETRESID(auio, 0);
 	    } else {		/* trouble, can't give anything to the user! */
 		/* even though he has given us a buffer, 
 		 * even though we have something to give us,
@@ -1153,7 +1169,7 @@ afs1_readdir(avc, auio, acred)
 	if (ode)
 	    DRelease(ode, 0);
 	ode = nde;
-	auio->afsio_offset = ((us + afs_dir_NameBlobs(nde->name)) << 5);
+	AFS_UIO_OFFSET(auio) = ((us + afs_dir_NameBlobs(nde->name)) << 5);
     }
     if (ode)
 	DRelease(ode, 0);
@@ -1167,6 +1183,7 @@ afs1_readdir(avc, auio, acred)
 #if	defined(AFS_HPUX_ENV) || defined(AFS_OSF_ENV)
     osi_FreeSmallSpace((char *)sdirEntry);
 #endif
+    AFS_DISCON_UNLOCK();
     afs_PutFakeStat(&fakestate);
     code = afs_CheckCode(code, &treq, 29);
     return code;
