@@ -14,16 +14,14 @@
  * afs_IsWired (DUX)
  * afsremove
  * afs_remove
- *
- * Local:
- * newname
+ * afs_newname
  *
  */
 #include <afsconfig.h>
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.31.2.4 2004/12/07 06:12:14 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.52.2.4.2.1 2008/05/23 14:25:35 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -106,30 +104,31 @@ afsremove(register struct vcache *adp, register struct dcache *tdc,
 	  register struct vcache *tvc, char *aname, struct AFS_UCRED *acred,
 	  struct vrequest *treqp)
 {
-    register afs_int32 code;
+    register afs_int32 code = 0;
     register struct conn *tc;
     struct AFSFetchStatus OutDirStatus;
     struct AFSVolSync tsync;
     XSTATS_DECLS;
-    do {
-	tc = afs_Conn(&adp->fid, treqp, SHARED_LOCK);
-	if (tc) {
-	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_REMOVEFILE);
-	    RX_AFS_GUNLOCK();
-	    code =
-		RXAFS_RemoveFile(tc->id, (struct AFSFid *)&adp->fid.Fid,
-				 aname, &OutDirStatus, &tsync);
-	    RX_AFS_GLOCK();
-	    XSTATS_END_TIME;
-	} else
-	    code = -1;
-    } while (afs_Analyze
-	     (tc, code, &adp->fid, treqp, AFS_STATS_FS_RPCIDX_REMOVEFILE,
-	      SHARED_LOCK, NULL));
+    
+    if (!AFS_IS_DISCONNECTED) {
+        do {
+	    tc = afs_Conn(&adp->fid, treqp, SHARED_LOCK);
+	    if (tc) {
+	        XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_REMOVEFILE);
+	        RX_AFS_GUNLOCK();
+	        code =
+		    RXAFS_RemoveFile(tc->id, (struct AFSFid *)&adp->fid.Fid,
+		  		     aname, &OutDirStatus, &tsync);
+	        RX_AFS_GLOCK();
+	        XSTATS_END_TIME;
+	    } else
+	        code = -1;
+        } while (afs_Analyze
+	         (tc, code, &adp->fid, treqp, AFS_STATS_FS_RPCIDX_REMOVEFILE,
+	          SHARED_LOCK, NULL));
+    }
 
     osi_dnlc_remove(adp, aname, tvc);
-    if (tvc)
-	afs_symhint_inval(tvc);	/* XXX: don't really need to be so extreme */
 
     if (code) {
 	if (tdc) {
@@ -171,7 +170,7 @@ afsremove(register struct vcache *adp, register struct dcache *tdc,
      * call FindVCache instead of GetVCache since if the file's really
      * gone, we won't be able to fetch the status info anyway.  */
     if (tvc) {
-#if	defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_BOZONLOCK_ENV
 	afs_BozonLock(&tvc->pvnLock, tvc);
 	/* Since afs_TryToSmush will do a pvn_vptrunc */
 #endif
@@ -185,7 +184,7 @@ afsremove(register struct vcache *adp, register struct dcache *tdc,
 		afs_TryToSmush(tvc, acred, 0);
 	}
 	ReleaseWriteLock(&tvc->lock);
-#if	defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_BOZONLOCK_ENV
 	afs_BozonUnlock(&tvc->pvnLock, tvc);
 #endif
 	afs_PutVCache(tvc);
@@ -193,8 +192,8 @@ afsremove(register struct vcache *adp, register struct dcache *tdc,
     return (0);
 }
 
-static char *
-newname(void)
+char *
+afs_newname(void)
 {
     char *name, *sp, *p = ".__afs";
     afs_int32 rd = afs_random() & 0xffff;
@@ -219,20 +218,11 @@ char *Tnam1;
 /* Note that we don't set CDirty here, this is OK because the unlink
  * RPC is called synchronously */
 int
-#ifdef	AFS_OSF_ENV
-afs_remove(ndp)
-     struct nameidata *ndp;
-{
-    register struct vcache *adp = VTOAFS(ndp->ni_dvp);
-    char *aname = ndp->ni_dent.d_name;
-    struct ucred *acred = ndp->ni_cred;
-#else				/* AFS_OSF_ENV */
 afs_remove(OSI_VC_ARG(adp), aname, acred)
      OSI_VC_DECL(adp);
      char *aname;
      struct AFS_UCRED *acred;
 {
-#endif
     struct vrequest treq;
     register struct dcache *tdc;
     struct VenusFid unlinkFid;
@@ -252,7 +242,6 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 
     if ((code = afs_InitReq(&treq, acred))) {
 #ifdef  AFS_OSF_ENV
-	afs_PutVCache(adp);
 	afs_PutVCache(tvc);
 #endif
 	return code;
@@ -263,7 +252,6 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
     if (code) {
 	afs_PutFakeStat(&fakestate);
 #ifdef  AFS_OSF_ENV
-	afs_PutVCache(adp);
 	afs_PutVCache(tvc);
 #endif
 	return code;
@@ -274,26 +262,17 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 	code = afs_DynrootVOPRemove(adp, acred, aname);
 	afs_PutFakeStat(&fakestate);
 #ifdef  AFS_OSF_ENV
-	afs_PutVCache(adp);
 	afs_PutVCache(tvc);
 #endif
 	return code;
     }
-#if 0
-    if (adp->mvstat == 2) {
-	afs_PutFakeStat(&fakestate);
-#ifdef  AFS_OSF_ENV
-	afs_PutVCache(adp);
-	afs_PutVCache(tvc);
-#endif
-	return EISDIR;
+    if (afs_IsDynrootMount(adp)) {
+	return ENOENT;
     }
-#endif
 
     if (strlen(aname) > AFSNAMEMAX) {
 	afs_PutFakeStat(&fakestate);
 #ifdef  AFS_OSF_ENV
-	afs_PutVCache(adp);
 	afs_PutVCache(tvc);
 #endif
 	return ENAMETOOLONG;
@@ -303,7 +282,6 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 #ifdef	AFS_OSF_ENV
     tvc = VTOAFS(ndp->ni_vp);	/* should never be null */
     if (code) {
-	afs_PutVCache(adp);
 	afs_PutVCache(tvc);
 	afs_PutFakeStat(&fakestate);
 	return afs_CheckCode(code, &treq, 22);
@@ -322,7 +300,6 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
       */
     if (adp->states & CRO) {
 #ifdef  AFS_OSF_ENV
-	afs_PutVCache(adp);
 	afs_PutVCache(tvc);
 #endif
 	code = EROFS;
@@ -330,6 +307,16 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 	return code;
     }
 
+    /* If we're running disconnected without logging, go no further... */
+    if (AFS_IS_DISCONNECTED && !AFS_IS_LOGGING) {
+#ifdef  AFS_OSF_ENV
+        afs_PutVCache(tvc);
+#endif
+        code = ENETDOWN;
+        afs_PutFakeStat(&fakestate);
+        return code;
+    }
+    
     tdc = afs_GetDCache(adp, (afs_size_t) 0, &treq, &offset, &len, 1);	/* test for error below */
     ObtainWriteLock(&adp->lock, 142);
     if (tdc)
@@ -378,6 +365,8 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
     if (tvc && osi_Active(tvc)) {
 	/* about to delete whole file, prefetch it first */
 	ReleaseWriteLock(&adp->lock);
+	if (tdc)
+	    ReleaseSharedLock(&tdc->lock);
 	ObtainWriteLock(&tvc->lock, 143);
 #if	defined(AFS_OSF_ENV)
 	afs_Wire(tvc, &treq);
@@ -386,33 +375,33 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 #endif
 	ReleaseWriteLock(&tvc->lock);
 	ObtainWriteLock(&adp->lock, 144);
+	/* Technically I don't think we need this back, but let's hold it 
+	   anyway; The "got" reference should actually be sufficient. */
+	if (tdc) 
+	    ObtainSharedLock(&tdc->lock, 640);
     }
 
     osi_dnlc_remove(adp, aname, tvc);
-    if (tvc)
-	afs_symhint_inval(tvc);
 
     Tadp1 = adp;
+#ifndef AFS_DARWIN80_ENV
     Tadpr = VREFCOUNT(adp);
+#endif
     Ttvc = tvc;
     Tnam = aname;
     Tnam1 = 0;
+#ifndef AFS_DARWIN80_ENV
     if (tvc)
 	Ttvcr = VREFCOUNT(tvc);
+#endif
 #ifdef	AFS_AIX_ENV
-    if (tvc && (VREFCOUNT(tvc) > 2) && tvc->opens > 0
-	&& !(tvc->states & CUnlinked))
+    if (tvc && VREFCOUNT_GT(tvc, 2) && tvc->opens > 0
+	&& !(tvc->states & CUnlinked)) {
 #else
-#ifdef AFS_DARWIN14_ENV
-    if (tvc && (VREFCOUNT(tvc) > 1 + DARWIN_REFBASE) && tvc->opens > 0
-	&& !(tvc->states & CUnlinked))
-#else
-    if (tvc && (VREFCOUNT(tvc) > 1) && tvc->opens > 0
-	&& !(tvc->states & CUnlinked))
+    if (tvc && VREFCOUNT_GT(tvc, 1) && tvc->opens > 0
+	&& !(tvc->states & CUnlinked)) {
 #endif
-#endif
-    {
-	char *unlname = newname();
+	char *unlname = afs_newname();
 
 	ReleaseWriteLock(&adp->lock);
 	if (tdc)
@@ -420,7 +409,12 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 	code = afsrename(adp, aname, adp, unlname, acred, &treq);
 	Tnam1 = unlname;
 	if (!code) {
+	    struct VenusFid *oldmvid = NULL;
+	    if (tvc->mvid) 
+		oldmvid = tvc->mvid;
 	    tvc->mvid = (struct VenusFid *)unlname;
+	    if (oldmvid)
+		osi_FreeSmallSpace(oldmvid);
 	    crhold(acred);
 	    if (tvc->uncred) {
 		crfree(tvc->uncred);
@@ -436,10 +430,12 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
     } else {
 	code = afsremove(adp, tdc, tvc, aname, acred, &treq);
     }
-#ifdef	AFS_OSF_ENV
-    afs_PutVCache(adp);
-#endif /* AFS_OSF_ENV */
     afs_PutFakeStat(&fakestate);
+#ifndef AFS_DARWIN80_ENV
+    /* we can't track by thread, it's not exported in the KPI; only do
+       this on !macos */
+    osi_Assert(!WriteLocked(&adp->lock) || (adp->lock.pid_writer != MyPidxx));
+#endif
     return code;
 }
 
@@ -459,12 +455,15 @@ afs_remunlink(register struct vcache *avc, register int doit)
     struct VenusFid dirFid;
     register struct dcache *tdc;
     afs_int32 code = 0;
-#ifdef AFS_DARWIN14_ENV
-    int oldref;
-#endif
 
     if (NBObtainWriteLock(&avc->lock, 423))
 	return 0;
+#if defined(AFS_DARWIN80_ENV)
+    if (vnode_get(AFSTOV(avc))) {
+	ReleaseWriteLock(&avc->lock);
+	return 0;
+    }
+#endif
 
     if (avc->mvid && (doit || (avc->states & CUnlinkedDel))) {
 	if ((code = afs_InitReq(&treq, avc->uncred))) {
@@ -478,30 +477,17 @@ afs_remunlink(register struct vcache *avc, register int doit)
 	    cred = avc->uncred;
 	    avc->uncred = NULL;
 
-#if defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN14_ENV)
-	    /* this is called by vrele (via VOP_INACTIVE) when the refcount
-	     * is 0. we can't just call VN_HOLD since vref will panic.
-	     * we can't just call osi_vnhold because a later AFS_RELE will call
-	     * vrele again, which will try to call VOP_INACTIVE again after
-	     * vn_locking the vnode. which would be fine except that our vrele
-	     * caller also locked the vnode... So instead, we just gimmick the
-	     * refcounts and hope nobody else can touch the file now */
-	    osi_Assert(VREFCOUNT(avc) == 0);
-	    VREFCOUNT_SET(avc, 1);
-#endif
+#if defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN80_ENV)
+	    VREF(AFSTOV(avc));
+#else
 	    VN_HOLD(AFSTOV(avc));
+#endif
 
 	    /* We'll only try this once. If it fails, just release the vnode.
 	     * Clear after doing hold so that NewVCache doesn't find us yet.
 	     */
 	    avc->states &= ~(CUnlinked | CUnlinkedDel);
 
-#ifdef AFS_DARWIN14_ENV
-	    if (VREFCOUNT(avc) < 4) {
-		oldref = 4 - VREFCOUNT(avc);
-		VREFCOUNT_SET(avc, 4);
-	    }
-#endif
 	    ReleaseWriteLock(&avc->lock);
 
 	    dirFid.Cell = avc->fid.Cell;
@@ -525,19 +511,11 @@ afs_remunlink(register struct vcache *avc, register int doit)
 	    }
 	    osi_FreeSmallSpace(unlname);
 	    crfree(cred);
-#ifdef AFS_DARWIN_ENV
-#ifndef AFS_DARWIN14_ENV
-	    osi_Assert(VREFCOUNT(avc) == 1);
-	    VREFCOUNT_SET(avc, 0);
-#else
-	    if (oldref) {
-		int newref = VREFCOUNT(avc) - oldref;
-		VREFCOUNT_SET(avc, newref);
-	    }
-#endif
-#endif
 	}
     } else {
+#if defined(AFS_DARWIN80_ENV)
+	vnode_put(AFSTOV(avc));
+#endif
 	ReleaseWriteLock(&avc->lock);
     }
 
