@@ -17,7 +17,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_init.c,v 1.28.2.2 2005/03/11 06:50:31 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_init.c,v 1.37.4.5.2.1 2008/05/23 14:25:34 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -27,7 +27,7 @@ RCSID
 /* Exported variables */
 struct osi_dev cacheDev;	/*Cache device */
 afs_int32 cacheInfoModTime;	/*Last time cache info modified */
-#if defined(AFS_OSF_ENV) || defined(AFS_DEC_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+#if defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
 struct mount *afs_cacheVfsp = 0;
 #elif defined(AFS_LINUX20_ENV)
 struct super_block *afs_cacheSBp = 0;
@@ -39,11 +39,15 @@ char *afs_sysname = 0;		/* So that superuser may change the
 				 * local value of @sys */
 char *afs_sysnamelist[MAXNUMSYSNAMES];	/* For support of a list of sysname */
 int afs_sysnamecount = 0;
+int afs_sysnamegen = 0;
 struct volume *Initialafs_freeVolList;
 int afs_memvolumes = 0;
 #if defined(AFS_XBSD_ENV)
 static struct vnode *volumeVnode;
 #endif
+#if defined(AFS_DISCON_ENV)
+afs_rwlock_t afs_discon_lock;
+#endif
 
 /*
  * Initialization order is important.  Must first call afs_CacheInit,
@@ -109,28 +113,11 @@ afs_CacheInit(afs_int32 astatSize, afs_int32 afiles, afs_int32 ablocks,
 
     LOCK_INIT(&afs_ftf, "afs_ftf");
     RWLOCK_INIT(&afs_xaxs, "afs_xaxs");
+#ifdef AFS_DISCON_ENV
+    RWLOCK_INIT(&afs_discon_lock, "afs_discon_lock");
+#endif
     osi_dnlc_init();
 
-
-#if	defined(AFS_AIX32_ENV) || defined(AFS_HPUX_ENV)
-    {
-	afs_int32 preallocs;
-
-	/*
-	 * We want to also reserve space for the gnode struct which is associated
-	 * with each vnode (vcache) one; we want to use the pinned pool for them   
-	 * since they're referenced at interrupt level.
-	 */
-	if (afs_stats_cmperf.SmallBlocksAlloced + astatSize < 3600)
-	    preallocs = astatSize;
-	else {
-	    preallocs = 3600 - afs_stats_cmperf.SmallBlocksAlloced;
-	    if (preallocs <= 0)
-		preallocs = 10;
-	}
-	osi_AllocMoreSSpace(preallocs);
-    }
-#endif
     /* 
      * create volume list structure 
      */
@@ -251,11 +238,7 @@ LookupInodeByPath(char *filename, ino_t * inode, struct vnode **fvpp)
     if (fvpp)
 	*fvpp = filevp;
     else {
-#if defined(AFS_DEC_ENV)
-	grele(filevp);
-#else
 	AFS_RELE(filevp);
-#endif
     }
 #endif /* AFS_LINUX22_ENV */
 
@@ -268,8 +251,12 @@ afs_InitCellInfo(char *afile)
     ino_t inode;
     int code;
 
+#ifdef AFS_CACHE_VNODE_PATH
+    return afs_cellname_init(AFS_CACHE_CELLS_INODE, code);
+#else
     code = LookupInodeByPath(afile, &inode, NULL);
     return afs_cellname_init(inode, code);
+#endif
 }
 
 /*
@@ -310,6 +297,8 @@ afs_InitVolumeInfo(char *afile)
      * it in the cache...
      */
     code = LookupInodeByPath(afile, &volumeInode, &volumeVnode);
+#elif defined(AFS_CACHE_VNODE_PATH)
+    volumeInode = AFS_CACHE_VOLUME_INODE;
 #else
     code = LookupInodeByPath(afile, &volumeInode, NULL);
 #endif
@@ -376,6 +365,8 @@ afs_InitCacheInfo(register char *afile)
 	struct statvfs st;
 #elif defined(AFS_DUX40_ENV)
 	struct nstatfs st;
+#elif defined(AFS_DARWIN80_ENV)
+	struct vfsstatfs st;
 #else
 	struct statfs st;
 #endif /* SUN56 */
@@ -405,6 +396,9 @@ afs_InitCacheInfo(register char *afile)
 	    VFS_STATFS(filevp->v_vfsp, &st);
 	    TO_KERNEL_SPACE();
 	}
+#elif defined(AFS_DARWIN80_ENV)
+        afs_cacheVfsp = vnode_mount(filevp);
+	if (afs_cacheVfsp && ((st = *(vfs_statfs(afs_cacheVfsp))),1))
 #elif defined(AFS_DARWIN_ENV)
 	if (!VFS_STATFS(filevp->v_mount, &st, current_proc()))
 #elif defined(AFS_FBSD50_ENV)
@@ -435,9 +429,15 @@ afs_InitCacheInfo(register char *afile)
 #if defined(AFS_SGI62_ENV) || defined(AFS_HAVE_VXFS) || defined(AFS_DARWIN_ENV)
     afs_InitDualFSCacheOps(filevp);
 #endif
-    cacheInode = afs_vnodeToInumber(filevp);
-    cacheDev.dev = afs_vnodeToDev(filevp);
+#ifndef AFS_CACHE_VNODE_PATH
+#ifndef AFS_DARWIN80_ENV
     afs_cacheVfsp = filevp->v_vfsp;
+#endif
+    cacheInode = afs_vnodeToInumber(filevp);
+#else
+    cacheInode = AFS_CACHE_ITEMS_INODE;
+#endif
+    cacheDev.dev = afs_vnodeToDev(filevp);
 #endif /* AFS_LINUX20_ENV */
     AFS_RELE(filevp);
 #endif /* AFS_LINUX22_ENV */
@@ -492,9 +492,7 @@ afs_ResourceInit(int preallocs)
     RWLOCK_INIT(&afs_xinterface, "afs_xinterface");
     LOCK_INIT(&afs_puttofileLock, "afs_puttofileLock");
 #ifndef AFS_FBSD_ENV
-#ifndef	AFS_AIX32_ENV
     LOCK_INIT(&osi_fsplock, "osi_fsplock");
-#endif
     LOCK_INIT(&osi_flplock, "osi_flplock");
 #endif
     RWLOCK_INIT(&afs_xconn, "afs_xconn");
@@ -511,15 +509,7 @@ afs_ResourceInit(int preallocs)
 	afs_sysname = afs_sysnamelist[0];
 	strcpy(afs_sysname, SYS_NAME);
 	afs_sysnamecount = 1;
-#if	defined(AFS_AIX32_ENV) || defined(AFS_HPUX_ENV)
-	{
-
-	    if ((preallocs > 256) && (preallocs < 3600))
-		afs_preallocs = preallocs;
-	    osi_AllocMoreSSpace(afs_preallocs);
-	    osi_AllocMoreMSpace(100);
-	}
-#endif
+	afs_sysnamegen++;
     }
 
     secobj = rxnull_NewServerSecurityObject();
@@ -691,7 +681,7 @@ shutdown_cache(void)
 void
 shutdown_vnodeops(void)
 {
-#if !defined(AFS_SGI_ENV) && !defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV)
+#if !defined(AFS_SGI_ENV) && !defined(AFS_SUN5_ENV)
     struct buf *afs_bread_freebp = 0;
 #endif
 
@@ -704,7 +694,7 @@ shutdown_vnodeops(void)
 #ifndef AFS_LINUX20_ENV
 	afs_rd_stash_i = 0;
 #endif
-#if !defined(AFS_SGI_ENV) && !defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV)
+#if !defined(AFS_SGI_ENV) && !defined(AFS_SUN5_ENV)
 	afs_bread_freebp = 0;
 #endif
 	shutdown_mariner();
