@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/bozo/bosserver.c,v 1.32.2.2 2006/06/20 20:37:10 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/bozo/bosserver.c,v 1.32.2.10 2008/03/10 22:32:32 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -21,6 +21,7 @@ RCSID
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #include <direct.h>
@@ -37,7 +38,8 @@ RCSID
 #include <rx/rx_globals.h>
 #include "bosint.h"
 #include "bnode.h"
-#include <afs/auth.h>
+#include "bosprototypes.h"
+#include <rx/rxkad.h>
 #include <afs/keys.h>
 #include <afs/ktime.h>
 #include <afs/afsutil.h>
@@ -47,16 +49,13 @@ RCSID
 #include <afs/afs_args.h>
 #endif
 
-
 #define BOZO_LWP_STACKSIZE	16000
 extern int BOZO_ExecuteRequest();
 extern int RXSTATS_ExecuteRequest();
 extern struct bnode_ops fsbnode_ops, dafsbnode_ops, ezbnode_ops, cronbnode_ops;
 
-void bozo_Log();
-
 struct afsconf_dir *bozo_confdir = 0;	/* bozo configuration dir */
-static char *bozo_pid;
+static PROCESS bozo_pid;
 struct rx_securityClass *bozo_rxsc[3];
 const char *bozo_fileName;
 FILE *bozo_logFile;
@@ -72,6 +71,7 @@ static afs_int32 nextDay;
 struct ktime bozo_nextRestartKT, bozo_nextDayKT;
 int bozo_newKTs;
 int rxBind = 0;
+int rxkadDisableDotCheck = 0;
 
 #define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
 afs_uint32 SHostAddrs[ADDRSPERSITE];
@@ -164,7 +164,7 @@ bozo_ReBozo()
 
 /* make sure a dir exists */
 static int
-MakeDir(register char *adir)
+MakeDir(const char *adir)
 {
     struct stat tstat;
     register afs_int32 code;
@@ -419,7 +419,7 @@ ReadBozoFile(char *aname)
 	code =
 	    bnode_Create(typep, instp, &tb, parms[0], parms[1], parms[2],
 			 parms[3], parms[4], notifier,
-			 goal ? BSTAT_NORMAL : BSTAT_SHUTDOWN);
+			 goal ? BSTAT_NORMAL : BSTAT_SHUTDOWN, 0);
 	if (code)
 	    goto fail;
 
@@ -514,8 +514,8 @@ bdrestart(register struct bnode *abnode, char *arock)
 
 #define	BOZO_MINSKIP 3600	/* minimum to advance clock */
 /* lwp to handle system restarts */
-static int
-BozoDaemon()
+static void *
+BozoDaemon(void *unused)
 {
     register afs_int32 now;
 
@@ -550,6 +550,7 @@ BozoDaemon()
 	    bnode_ApplyInstance(bdrestart, 0);
 	}
     }
+    return NULL;
 }
 
 #ifdef AFS_AIX32_ENV
@@ -720,7 +721,6 @@ main(int argc, char **argv, char **envp)
     register afs_int32 code;
     struct afsconf_dir *tdir;
     int noAuth = 0;
-    struct ktc_encryptionKey tkey;
     int i;
     char namebuf[AFSDIR_PATH_MAX];
     int rxMaxMTU = -1;
@@ -822,6 +822,9 @@ main(int argc, char **argv, char **envp)
 	else if (strcmp(argv[code], "-rxbind") == 0) {
 	    rxBind = 1;
 	}
+	else if (strcmp(argv[code], "-allow-dotted-principals") == 0) {
+	    rxkadDisableDotCheck = 1;
+	}
 	else if (!strcmp(argv[i], "-rxmaxmtu")) {
 	    if ((i + 1) >= argc) {
 		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
@@ -830,7 +833,7 @@ main(int argc, char **argv, char **envp)
 	    rxMaxMTU = atoi(argv[++i]);
 	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) || 
 		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d% invalid; must be between %d-%d\n",
+		printf("rxMaxMTU %d invalid; must be between %d-%d\n",
 			rxMaxMTU, RX_MIN_PACKET_SIZE, 
 			RX_MAX_PACKET_DATA_SIZE);
 		exit(1);
@@ -873,14 +876,14 @@ main(int argc, char **argv, char **envp)
 #ifndef AFS_NT40_ENV
 	    printf("Usage: bosserver [-noauth] [-log] "
 		   "[-auditlog <log path>] "
-		   "[-rxmaxmtu <bytes>] [-rxbind] "
+		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-syslog[=FACILITY]] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-nofork] " "[-help]\n");
 #else
 	    printf("Usage: bosserver [-noauth] [-log] "
 		   "[-auditlog <log path>] "
-		   "[-rxmaxmtu <bytes>] [-rxbind] "
+		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-help]\n");
 #endif
@@ -1016,7 +1019,6 @@ main(int argc, char **argv, char **envp)
 
     /* opened the cell databse */
     bozo_confdir = tdir;
-    code = afsconf_GetKey(tdir, 999, &tkey);
 
     /* allow super users to manage RX statistics */
     rx_SetRxStatUserOk(bozo_rxstat_userok);
@@ -1064,6 +1066,11 @@ main(int argc, char **argv, char **envp)
     rx_SetMinProcs(tservice, 2);
     rx_SetMaxProcs(tservice, 4);
     rx_SetStackSize(tservice, BOZO_LWP_STACKSIZE);	/* so gethostbyname works (in cell stuff) */
+    if (rxkadDisableDotCheck) {
+        rx_SetSecurityConfiguration(tservice, RXS_CONFIG_FLAGS,
+                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK, 
+                                    NULL);
+    }
 
     tservice =
 	rx_NewServiceHost(host, 0, RX_STATS_SERVICE_ID, "rpcstats", bozo_rxsc,
@@ -1071,17 +1078,21 @@ main(int argc, char **argv, char **envp)
     rx_SetMinProcs(tservice, 2);
     rx_SetMaxProcs(tservice, 4);
     rx_StartServer(1);		/* donate this process */
+    return 0;
 }
 
 void
-bozo_Log(char *a, char *b, char *c, char *d, char *e, char *f)
+bozo_Log(char *format, ...)
 {
     char tdate[27];
     time_t myTime;
+    va_list ap;
+
+    va_start(ap, format);
 
     if (DoSyslog) {
 #ifndef AFS_NT40_ENV
-	syslog(LOG_INFO, a, b, c, d, e, f);
+        vsyslog(LOG_INFO, format, ap);
 #endif
     } else {
 	myTime = time(0);
@@ -1095,11 +1106,11 @@ bozo_Log(char *a, char *b, char *c, char *d, char *e, char *f)
 	    printf("bosserver: WARNING: problem with %s\n",
 		   AFSDIR_SERVER_BOZLOG_FILEPATH);
 	    printf("%s ", tdate);
-	    printf(a, b, c, d, e, f);
+	    vprintf(format, ap);
 	    fflush(stdout);
 	} else {
 	    fprintf(bozo_logFile, "%s ", tdate);
-	    fprintf(bozo_logFile, a, b, c, d, e, f);
+	    vfprintf(bozo_logFile, format, ap);
 
 	    /* close so rm BosLog works */
 	    fclose(bozo_logFile);
