@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/butc/recoverDb.c,v 1.13 2005/07/11 03:22:20 shadow Exp $");
+    ("$Header: /cvs/openafs/src/butc/recoverDb.c,v 1.13.4.4 2008/03/10 22:32:33 shadow Exp $");
 
 #include <stdio.h>
 #ifdef AFS_NT40_ENV
@@ -24,6 +24,7 @@ RCSID
 #include <strings.h>
 #endif
 #include <sys/types.h>
+#include <string.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <lwp.h>
@@ -51,12 +52,13 @@ struct tapeScanInfo {
 extern struct tapeConfig globalTapeConfig;
 extern struct deviceSyncNode *deviceLatch;
 
+static readDump();
+
 /* PrintDumpLabel
  *	print out the tape (dump) label.
  */
-
-PrintDumpLabel(labelptr)
-     struct butm_tapeLabel *labelptr;
+void
+PrintDumpLabel(struct butm_tapeLabel *labelptr)
 {
     char tapeName[BU_MAXTAPELEN + 32];
     time_t t;
@@ -87,9 +89,8 @@ PrintDumpLabel(labelptr)
 /* PrintVolumeHeader
  *	print the contents of a volume header. 
  */
-static
-PrintVolumeHeader(volHeader)
-     struct volumeHeader *volHeader;
+static void
+PrintVolumeHeader(struct volumeHeader *volHeader)
 {
     time_t t;
 
@@ -115,8 +116,7 @@ PrintVolumeHeader(volHeader)
  */
 
 afs_int32
-Ask(st)
-     char *st;
+Ask(char *st)
 {
     int response;
 
@@ -152,14 +152,8 @@ Ask(st)
  */
 #define BIGCHUNK 102400
 
-static
-scanVolData(taskId, curTapePtr, tapeVersion, volumeHeader, volumeTrailer,
-	    bytesRead)
-     afs_int32 taskId;
-     struct butm_tapeInfo *curTapePtr;
-     afs_int32 tapeVersion;
-     struct volumeHeader *volumeHeader, *volumeTrailer;
-     afs_uint32 *bytesRead;
+static int
+scanVolData(afs_int32 taskId, struct butm_tapeInfo *curTapePtr, afs_int32 tapeVersion, struct volumeHeader *volumeHeader, struct volumeHeader *volumeTrailer, afs_uint32 *bytesRead)
 {
     afs_int32 headBytes, tailBytes;
     char *block = NULL;
@@ -290,8 +284,7 @@ scanVolData(taskId, curTapePtr, tapeVersion, volumeHeader, volumeTrailer,
  */
 
 char *
-nextTapeLabel(prevTapeName)
-     char *prevTapeName;
+nextTapeLabel(char *prevTapeName)
 {
     char *prevdot;
     char *retval;
@@ -331,11 +324,8 @@ nextTapeLabel(prevTapeName)
 
 afs_int32 RcreateDump();
 
-static
-readDump(taskId, tapeInfoPtr, scanInfoPtr)
-     afs_uint32 taskId;
-     struct butm_tapeInfo *tapeInfoPtr;
-     struct tapeScanInfo *scanInfoPtr;
+static int
+readDump(afs_uint32 taskId, struct butm_tapeInfo *tapeInfoPtr, struct tapeScanInfo *scanInfoPtr)
 {
     int moreTapes = 1;
     afs_int32 nbytes, flags, seq;
@@ -577,10 +567,8 @@ readDump(taskId, tapeInfoPtr, scanInfoPtr)
  * try to read that dump too.
  * The first tape label is the first dumpLabel.
  */
-readDumps(taskId, tapeInfoPtr, scanInfoPtr)
-     afs_uint32 taskId;
-     struct butm_tapeInfo *tapeInfoPtr;
-     struct tapeScanInfo *scanInfoPtr;
+int
+readDumps(afs_uint32 taskId, struct butm_tapeInfo *tapeInfoPtr, struct tapeScanInfo *scanInfoPtr)
 {
     afs_int32 code, c;
 
@@ -611,13 +599,7 @@ readDumps(taskId, tapeInfoPtr, scanInfoPtr)
 }
 
 afs_int32
-getScanTape(taskId, tapeInfoPtr, tname, tapeId, prompt, tapeLabelPtr)
-     afs_int32 taskId;
-     struct butm_tapeInfo *tapeInfoPtr;
-     char *tname;
-     afs_int32 tapeId;
-     int prompt;
-     struct butm_tapeLabel *tapeLabelPtr;
+getScanTape(afs_int32 taskId, struct butm_tapeInfo *tapeInfoPtr, char *tname, afs_int32 tapeId, int prompt, struct butm_tapeLabel *tapeLabelPtr)
 {
     afs_int32 code = 0;
     int tapecount = 1;
@@ -710,9 +692,11 @@ getScanTape(taskId, tapeInfoPtr, tname, tapeId, prompt, tapeLabelPtr)
  *	
  */
 
-ScanDumps(ptr)
-     struct scanTapeIf *ptr;
+void *
+ScanDumps(void *param)
 {
+    struct scanTapeIf *ptr = (struct scanTapeIf *)param;
+    
     struct butm_tapeInfo curTapeInfo;
     struct tapeScanInfo tapeScanInfo;
     afs_uint32 taskId;
@@ -769,7 +753,7 @@ ScanDumps(ptr)
     free(ptr);
     setStatus(taskId, TASK_DONE);
     LeaveDeviceQueue(deviceLatch);
-    return (code);
+    return (void *)(code);
 }
 
 
@@ -778,9 +762,8 @@ ScanDumps(ptr)
  *	0 - not ok
  *	1 - ok
  */
-validatePath(labelptr, pathptr)
-     struct butm_tapeLabel *labelptr;
-     char *pathptr;
+int
+validatePath(struct butm_tapeLabel *labelptr, char *pathptr)
 {
     char *up, *tp;
     char tapeName[BU_MAXTAPELEN];
@@ -828,8 +811,7 @@ validatePath(labelptr, pathptr)
  */
 
 char *
-volumesetNamePtr(ptr)
-     char *ptr;
+volumesetNamePtr(char *ptr)
 {
     static char vsname[BU_MAXUNAMELEN];
     char *dotPtr;
@@ -850,8 +832,7 @@ volumesetNamePtr(ptr)
 }
 
 char *
-extractDumpName(ptr)
-     char *ptr;
+extractDumpName(char *ptr)
 {
     static char dname[BU_MAXTAPELEN];
     char *dotPtr;
@@ -882,8 +863,8 @@ extractDumpName(ptr)
  *	-1 - error, couldn't extract sequence number
  */
 
-extractTapeSeq(tapename)
-     char *tapename;
+int
+extractTapeSeq(char *tapename)
 {
     char *sptr;
 
@@ -899,8 +880,7 @@ extractTapeSeq(tapename)
  *   a database tape or not.
  */
 int
-databaseTape(tapeName)
-     char *tapeName;
+databaseTape(char *tapeName)
 {
     char *sptr;
     int c;
@@ -917,9 +897,7 @@ databaseTape(tapeName)
 }
 
 afs_int32
-RcreateDump(tapeScanInfoPtr, volHeaderPtr)
-     struct tapeScanInfo *tapeScanInfoPtr;
-     struct volumeHeader *volHeaderPtr;
+RcreateDump(struct tapeScanInfo *tapeScanInfoPtr, struct volumeHeader *volHeaderPtr)
 {
     afs_int32 code;
     struct butm_tapeLabel *dumpLabelPtr = &tapeScanInfoPtr->dumpLabel;
