@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/log/kseal.c,v 1.7 2003/07/15 23:15:40 shadow Exp $");
+    ("$Header: /cvs/openafs/src/log/kseal.c,v 1.8.4.1 2007/10/30 15:16:42 shadow Exp $");
 
 #include <sys/types.h>
 #include <netinet/in.h>
@@ -19,13 +19,7 @@ RCSID
 #include <signal.h>
 #endif
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #include <afs/cellconfig.h>
 #include <afs/afsutil.h>
@@ -37,9 +31,8 @@ RCSID
 
 #include "AFS_component_version_number.c"
 
-main(argc, argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char **argv)
 {
     struct ktc_token token;
     struct ktc_principal sname;
