@@ -1,4 +1,4 @@
-/* $Id: pt_util.c,v 1.11.4.2 2007/10/31 04:09:32 shadow Exp $ */
+/* $Id: pt_util.c,v 1.11.4.4 2008/03/27 18:40:12 shadow Exp $ */
 
 /*
  *
@@ -23,7 +23,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/pt_util.c,v 1.11.4.2 2007/10/31 04:09:32 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/pt_util.c,v 1.11.4.4 2008/03/27 18:40:12 shadow Exp $");
 
 #include <afs/cmd.h>		/*Command line parsing */
 #include <errno.h>
@@ -33,6 +33,7 @@ RCSID
 #include <ubik.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
+#include <afs/com_err.h>
 #include "ptint.h"
 #include "ptserver.h"
 #include "pterror.h"
@@ -195,8 +196,9 @@ CommandProc(register struct cmd_syndesc *a_as, void *arock)
 	fprintf(stderr, "pt_util: %s: Bad UBIK_MAGIC. Is %x should be %x\n",
 		pfile, ntohl(uh->magic), UBIK_MAGIC);
     memcpy(&uv, &uh->version, sizeof(struct ubik_version));
-    if (wflag && uv.epoch == 0 && uv.counter == 0) {
-	uv.epoch = 2;		/* a ubik version of 0 or 1 has special meaning */
+
+    if (wflag && ntohl(uv.epoch) == 0 && ntohl(uv.counter) == 0) {
+	uv.epoch = htonl(2); /* a ubik version of 0 or 1 has special meaning */
 	memcpy(&uh->version, &uv, sizeof(struct ubik_version));
 	lseek(dbase_fd, 0, SEEK_SET);
 	if (write(dbase_fd, buffer, HDRSIZE) < 0) {
@@ -205,6 +207,11 @@ CommandProc(register struct cmd_syndesc *a_as, void *arock)
 	    exit(1);
 	}
     }
+
+    /* Now that any writeback is done, swap these */
+    uv.epoch = ntohl(uv.epoch);
+    uv.counter = ntohl(uv.counter);
+
     fprintf(stderr, "Ubik Version is: %d.%d\n", uv.epoch, uv.counter);
     if (read(dbase_fd, &prh, sizeof(struct prheader)) < 0) {
 	fprintf(stderr, "pt_util: error reading %s: %s\n", pfile,
@@ -328,6 +335,10 @@ CommandProc(register struct cmd_syndesc *a_as, void *arock)
 	exit(1);
     }
     uh = (struct ubik_hdr *)buffer;
+
+    uh->version.epoch = ntohl(uh->version.epoch);
+    uh->version.counter = ntohl(uh->version.counter);
+
     if ((uh->version.epoch != uv.epoch)
 	|| (uh->version.counter != uv.counter)) {
 	fprintf(stderr,
