@@ -15,7 +15,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_kcommon.c,v 1.56.2.1 2006/07/31 15:09:09 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_kcommon.c,v 1.56.2.8 2008/04/09 16:40:00 shadow Exp $");
 
 #include "rx/rx_kcommon.h"
 
@@ -124,6 +124,7 @@ rxi_GetUDPSocket(u_short port)
     return rxi_GetHostUDPSocket(htonl(INADDR_ANY), port);
 }
 
+#if !defined(AFS_LINUX26_ENV)
 void
 osi_Panic(msg, a1, a2, a3)
      char *msg;
@@ -132,7 +133,7 @@ osi_Panic(msg, a1, a2, a3)
 	msg = "Unknown AFS panic";
 
     printf(msg, a1, a2, a3);
-#ifdef AFS_LINUX24_ENV
+#ifdef AFS_LINUX20_ENV
     * ((char *) 0) = 0; 
 #else
     panic(msg);
@@ -261,12 +262,19 @@ osi_AssertFailK(const char *expr, const char *file, int line)
 
     osi_Panic(buf);
 }
+#else
+void
+osi_AssertFailK(const char *expr, const char *file, int line)
+{
+    printk(KERN_CRIT "assertion failed: %s, file: %s, line: %d\n", expr, file, line);
+}
+#endif
 
 #ifndef UKERNEL
 /* This is the server process request loop. Kernel server
  * processes never become listener threads */
-void
-rx_ServerProc(void)
+void *
+rx_ServerProc(void *unused)
 {
     int threadID;
 
@@ -283,6 +291,8 @@ rx_ServerProc(void)
 #ifdef RX_ENABLE_LOCKS
     AFS_GLOCK();
 #endif /* RX_ENABLE_LOCKS */
+
+    return NULL;
 }
 #endif /* !UKERNEL */
 
@@ -512,7 +522,7 @@ shutdown_rxkernel(void)
 int
 rxi_GetcbiInfo(void)
 {
-    int i, j, different = 0;
+    int i, j, different = 0, num = ADDRSPERSITE;
     int rxmtu, maxmtu;
     afs_uint32 ifinaddr;
     afs_uint32 addrs[ADDRSPERSITE];
@@ -521,7 +531,9 @@ rxi_GetcbiInfo(void)
     memset((void *)addrs, 0, sizeof(addrs));
     memset((void *)mtus, 0, sizeof(mtus));
 
-    for (i = 0; i < afs_cb_interface.numberOfInterfaces; i++) {
+    if (afs_cb_interface.numberOfInterfaces < num)
+	num = afs_cb_interface.numberOfInterfaces;
+    for (i = 0; i < num; i++) {
 	if (!afs_cb_interface.mtu[i])
 	    afs_cb_interface.mtu[i] = htonl(1500);
 	rxmtu = (ntohl(afs_cb_interface.mtu[i]) - RX_IPUDP_SIZE);
@@ -627,11 +639,13 @@ rxi_GetIFInfo(void)
     afs_uint32 ifinaddr;
 #if defined(AFS_DARWIN80_ENV)
     errno_t t;
-    int cnt=0;
-    ifaddr_t *ifads, ifad;
-    register ifnet_t ifn;
+    unsigned int count;
+    int cnt=0, m, j;
+    ifaddr_t *ifads;
+    ifnet_t *ifn;
     struct sockaddr sout;
     struct sockaddr_in *sin;
+    struct in_addr pin;
 #else
     struct ifaddr *ifad;	/* ifnet points to a if_addrlist of ifaddrs */
     register struct ifnet *ifn;
@@ -641,31 +655,38 @@ rxi_GetIFInfo(void)
     memset(mtus, 0, sizeof(mtus));
 
 #if defined(AFS_DARWIN80_ENV)
-    t = ifnet_get_address_list_family(NULL, &ifads, AF_INET);
-    if (t == 0) {
-	rxmtu = ifnet_mtu(ifn) - RX_IPUDP_SIZE;
-	while((ifads[cnt] != NULL) && cnt < ADDRSPERSITE) {
-	    t = ifaddr_address(ifads[cnt], &sout, sizeof(sout));
-	    sin = (struct sockaddr_in *)&sout;
-	    ifinaddr = ntohl(sin->sin_addr.s_addr);
-	    if (myNetAddrs[i] != ifinaddr) {
-		different++;
+    if (!ifnet_list_get(AF_INET, &ifn, &count)) {
+	for (m = 0; m < count; m++) {
+	    if (!ifnet_get_address_list(ifn[m], &ifads)) {
+		for (j = 0; ifads[j] != NULL && cnt < ADDRSPERSITE; j++) {
+		    if ((t = ifaddr_address(ifads[j], &sout, sizeof(struct sockaddr))) == 0) {
+			sin = (struct sockaddr_in *)&sout;
+			rxmtu = ifnet_mtu(ifaddr_ifnet(ifads[j])) - RX_IPUDP_SIZE;
+			ifinaddr = ntohl(sin->sin_addr.s_addr);
+			if (myNetAddrs[i] != ifinaddr) {
+			    different++;
+			}
+			mtus[i] = rxmtu;
+			rxmtu = rxi_AdjustIfMTU(rxmtu);
+			maxmtu =
+			    rxmtu * rxi_nRecvFrags +
+			    ((rxi_nRecvFrags - 1) * UDP_HDR_SIZE);
+			maxmtu = rxi_AdjustMaxMTU(rxmtu, maxmtu);
+			addrs[i++] = ifinaddr;
+			if ((ifinaddr != 0x7f000001) && 
+			    (maxmtu > rx_maxReceiveSize)) {
+			    rx_maxReceiveSize = 
+				MIN(RX_MAX_PACKET_SIZE, maxmtu);
+			    rx_maxReceiveSize =
+				MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
+			}
+			cnt++;
+		    }
+		}
+		ifnet_free_address_list(ifads);
 	    }
-	    mtus[i] = rxmtu;
-	    rxmtu = rxi_AdjustIfMTU(rxmtu);
-	    maxmtu =
-		rxmtu * rxi_nRecvFrags +
-		((rxi_nRecvFrags - 1) * UDP_HDR_SIZE);
-	    maxmtu = rxi_AdjustMaxMTU(rxmtu, maxmtu);
-	    addrs[i++] = ifinaddr;
-	    if ((ifinaddr != 0x7f000001) && (maxmtu > rx_maxReceiveSize)) {
-		rx_maxReceiveSize = MIN(RX_MAX_PACKET_SIZE, maxmtu);
-		rx_maxReceiveSize =
-		    MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
-	    }
-	    cnt++;
 	}
-	ifnet_free_address_list(ifads);
+	ifnet_list_free(ifn);
     }
 #else
 #if defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
@@ -721,10 +742,10 @@ rxi_GetIFInfo(void)
     rx_maxJumboRecvSize = MAX(rx_maxJumboRecvSize, rx_maxReceiveSize);
 
     if (different) {
-	int j;
-	for (j = 0; j < i; j++) {
-	    myNetMTUs[j] = mtus[j];
-	    myNetAddrs[j] = addrs[j];
+	int l;
+	for (l = 0; l < i; l++) {
+	    myNetMTUs[l] = mtus[l];
+	    myNetAddrs[l] = addrs[l];
 	}
     }
     return different;
@@ -1069,8 +1090,12 @@ afs_rxevent_daemon(void)
 	if (afs_termState == AFSOP_STOP_RXEVENT) {
 #ifdef RXK_LISTENER_ENV
 	    afs_termState = AFSOP_STOP_RXK_LISTENER;
+#else
+#ifdef AFS_SUN510_ENV
+	    afs_termState = AFSOP_STOP_NETIF;
 #else
 	    afs_termState = AFSOP_STOP_COMPLETE;
+#endif
 #endif
 	    osi_rxWakeup(&afs_termState);
 	    return;
@@ -1183,6 +1208,9 @@ rxk_ReadPacket(osi_socket so, struct rx_packet *p, int *host, int *port)
  * OS's socket receive routine returns as a result of a signal.
  */
 int rxk_ListenerPid;		/* Used to signal process to wakeup at shutdown */
+#ifdef AFS_LINUX20_ENV
+struct task_struct *rxk_ListenerTask;
+#endif
 
 #ifdef AFS_SUN5_ENV
 /*
@@ -1212,6 +1240,7 @@ rxk_Listener(void)
 
 #ifdef AFS_LINUX20_ENV
     rxk_ListenerPid = current->pid;
+    rxk_ListenerTask = current;
 #endif
 #ifdef AFS_SUN5_ENV
     rxk_ListenerPid = 1;	/* No PID, just a flag that we're alive */
@@ -1244,11 +1273,19 @@ rxk_Listener(void)
     AFS_GLOCK();
 #endif /* RX_ENABLE_LOCKS */
     if (afs_termState == AFSOP_STOP_RXK_LISTENER) {
+#ifdef AFS_SUN510_ENV
+	afs_termState = AFSOP_STOP_NETIF;
+#else
 	afs_termState = AFSOP_STOP_COMPLETE;
+#endif
 	osi_rxWakeup(&afs_termState);
     }
     rxk_ListenerPid = 0;
-#if defined(AFS_LINUX22_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_LINUX20_ENV
+    rxk_ListenerTask = 0;
+    osi_rxWakeup(&rxk_ListenerTask);
+#endif
+#if defined(AFS_SUN5_ENV)
     osi_rxWakeup(&rxk_ListenerPid);
 #endif
 #ifdef AFS_SUN5_ENV
