@@ -14,6 +14,9 @@
 extern void rx_SetEpoch(afs_uint32 epoch);
 extern int rx_Init(u_int port);
 extern int rx_InitHost(u_int host, u_int port);
+#ifdef AFS_NT40_ENV
+extern void rx_DebugOnOff(int on);
+#endif
 #ifndef KERNEL
 extern void rxi_StartServerProcs(int nExistingProcs);
 #endif
@@ -43,6 +46,15 @@ extern struct rx_service *rx_NewService(u_short port, u_short serviceId,
 					afs_int32(*serviceProc) (struct
 								 rx_call *
 								 acall));
+extern struct rx_service *rx_NewServiceHost(afs_uint32 host, u_short port, 
+					    u_short serviceId,
+					    char *serviceName,
+					    struct rx_securityClass
+					    **securityObjects,
+					    int nSecurityObjects,
+					    afs_int32(*serviceProc) (struct
+								     rx_call *
+								     acall));
 extern void rxi_ServerProc(int threadID, struct rx_call *newcall,
 			   osi_socket * socketp);
 extern void rx_WakeupServerProcs(void);
@@ -51,9 +63,9 @@ extern struct rx_call *rx_GetCall(int tno, struct rx_service *cur_service,
 extern void rx_SetArrivalProc(register struct rx_call *call,
 			      register void (*proc) (register struct rx_call *
 						    call,
-						    register VOID * mh,
+						    register void * mh,
 						    register int index),
-			      register VOID * handle, register int arg);
+			      register void * handle, register int arg);
 extern afs_int32 rx_EndCall(register struct rx_call *call, afs_int32 rc);
 extern void rx_Finalize(void);
 extern void rxi_PacketsUnWait(void);
@@ -71,6 +83,8 @@ extern void rxi_FreeCall(register struct rx_call *call);
 
 extern char *rxi_Alloc(register size_t size);
 extern void rxi_Free(void *addr, register size_t size);
+extern void rxi_SetPeerMtu(register afs_uint32 host, register afs_uint32 port,
+            int mtu);
 extern struct rx_peer *rxi_FindPeer(register afs_uint32 host,
 				    register u_short port,
 				    struct rx_peer *origPeer, int create);
@@ -238,10 +252,13 @@ extern int rx_RxStatUserOk(struct rx_call *call);
 /* old style till varargs */
 #if 0
 void
-  rxi_DebugPrint(char *format, int a1, int a2, int a3, int a4, int a5, int a6,
-		 int a7, int a8, int a9, int a10, int a11, int a12, int a13,
-		 int a14, int a15);
+rxi_DebugPrint(char *format, int a1, int a2, int a3, int a4, int a5, int a6,
+               int a7, int a8, int a9, int a10, int a11, int a12, int a13,
+	       int a14, int a15);
+void
+rxi_DebugInit(void);
 #else
+void rxi_DebugInit();
 void rxi_DebugPrint();
 #endif
 
@@ -289,6 +306,11 @@ extern struct rxevent *rxevent_Post(struct clock *when, void (*func) (),
 				    void *arg, void *arg1);
 extern struct rxevent *rxevent_Post2(struct clock *when, void (*func) (),
 				    void *arg, void *arg1, int arg2);
+extern struct rxevent *rxevent_PostNow(struct clock *when, struct clock *now,
+				       void (*func) (), void *arg, void *arg1);
+extern struct rxevent *rxevent_PostNow2(struct clock *when, struct clock *now,
+					void (*func) (), void *arg, 
+					void *arg1, int arg2);
 #endif
 extern void shutdown_rxevent(void);
 extern struct rxepoch *rxepoch_Allocate(struct clock *when);
@@ -309,10 +331,12 @@ extern afs_int32 rxi_getaddr(void);
 
 
 /* rx_kcommon.c */
-extern int (*rxk_PacketArrivalProc) (register struct rx_packet * ahandle,
-				     register struct sockaddr_in * afrom,
-				     char *arock, afs_int32 asize);
-extern int (*rxk_GetPacketProc) (char **ahandle, int asize);
+struct socket;
+extern int (*rxk_PacketArrivalProc) (struct rx_packet * ahandle,
+				     struct sockaddr_in * afrom,
+				     struct socket *arock,
+				     afs_int32 asize);
+extern int (*rxk_GetPacketProc) (struct rx_packet **ahandle, int asize);
 extern afs_int32 afs_termState;
 extern int rxk_initDone;
 
@@ -320,7 +344,14 @@ extern int rxk_DelPort(u_short aport);
 extern void rxk_shutdownPorts(void);
 extern osi_socket rxi_GetUDPSocket(u_short port);
 extern osi_socket rxi_GetHostUDPSocket(u_int host, u_short port);
-extern void osi_Panic();
+#if defined(KERNEL) && defined(AFS_LINUX26_ENV)
+#define osi_Panic(msg...) do { printk(KERN_CRIT "openafs: " msg); BUG(); } while (0)
+#undef osi_Assert
+#define osi_Assert(expr) \
+    do { if (!(expr)) { osi_AssertFailK(#expr, __FILE__, __LINE__); BUG(); } } while (0)
+#else
+extern void osi_Panic();       /* leave without args till stdarg rewrite */
+#endif
 extern int osi_utoa(char *buf, size_t len, unsigned long val);
 extern void rxi_InitPeerParams(register struct rx_peer *pp);
 extern void shutdown_rxkernel(void);
@@ -335,26 +366,29 @@ extern int rxi_GetIFInfo(void);
 extern int rxk_FreeSocket(register struct socket *asocket);
 #endif
 #ifndef AFS_NT40_ENV
-extern struct osi_socket *rxk_NewSocket(short aport);
+extern osi_socket *rxk_NewSocket(short aport);
 #endif
 #endif
 extern int rxk_ReadPacket(osi_socket so, struct rx_packet *p, int *host,
 			  int *port);
 #ifdef UKERNEL
-extern void rx_ServerProc(void);
+extern void *rx_ServerProc(void *);
 #endif
 extern void osi_AssertFailK(const char *expr, const char *file, int line);
 extern void rxk_ListenerProc(void);
 extern void rxk_Listener(void);
 #ifndef UKERNEL
 extern void afs_rxevent_daemon(void);
+#endif
+#if defined(AFS_DARWIN80_ENV) && defined(KERNEL)
+extern ifnet_t rxi_FindIfnet(afs_uint32 addr, afs_uint32 * maskp);
+#else
 extern struct ifnet *rxi_FindIfnet(afs_uint32 addr, afs_uint32 * maskp);
 #endif
 extern void osi_StopListener(void);
 
-
 /* ARCH/rx_kmutex.c */
-#ifdef CONFIG_SMP
+#if defined(KERNEL) && defined(AFS_LINUX20_ENV)
 extern void afs_mutex_init(afs_kmutex_t * l);
 extern void afs_mutex_enter(afs_kmutex_t * l);
 extern int afs_mutex_tryenter(afs_kmutex_t * l);
@@ -374,6 +408,11 @@ extern int osi_NetSend(osi_socket asocket, struct sockaddr_in *addr,
 #endif
 extern int osi_NetReceive(osi_socket so, struct sockaddr_in *addr,
 			  struct iovec *dvec, int nvecs, int *lengthp);
+#if defined(KERNEL) && defined(AFS_SUN510_ENV)
+extern void osi_StartNetIfPoller(void);
+extern void osi_NetIfPoller(void);
+extern struct afs_ifinfo afsifinfo[ADDRSPERSITE];
+#endif
 extern void osi_StopListener(void);
 extern int rxi_FindIfMTU(afs_uint32 addr);
 #ifndef RXK_LISTENER_ENV
@@ -387,7 +426,6 @@ extern void afs_rxevent_daemon(void);
 
 
 /* rx_lwp.c */
-extern void rx_ServerProc(void);
 extern void rxi_Sleep(void *addr);
 extern void rxi_Delay(int seconds);
 extern void rxi_InitializeThreadSupport(void);
@@ -397,11 +435,11 @@ extern void rxi_StopListener(void);
 extern void rxi_ReScheduleEvents(void);
 #endif
 extern void rxi_InitializeThreadSupport(void);
-extern void rxi_StartServerProc(void (*proc) (void), int stacksize);
+extern void rxi_StartServerProc(void *(*proc) (void *), int stacksize);
 extern void rxi_StartListener(void);
-extern void rx_ServerProc(void);
+extern void *rx_ServerProc(void *);
 extern int rxi_Listen(osi_socket sock);
-extern int rxi_Recvmsg(int socket, struct msghdr *msg_p, int flags);
+extern int rxi_Recvmsg(osi_socket socket, struct msghdr *msg_p, int flags);
 extern int rxi_Sendmsg(osi_socket socket, struct msghdr *msg_p, int flags);
 
 
@@ -421,7 +459,7 @@ extern struct multi_handle *multi_Init(struct rx_connection **conns,
 				       register int nConns);
 extern int multi_Select(register struct multi_handle *mh);
 extern void multi_Ready(register struct rx_call *call,
-			register VOID *mh, register int index);
+			register void *mh, register int index);
 extern void multi_Finalize(register struct multi_handle *mh);
 extern void multi_Finalize_Ignore(register struct multi_handle *mh);
 
@@ -439,7 +477,6 @@ extern afs_int32 rx_SlowReadPacket(struct rx_packet *packet,
 				   unsigned int offset, int resid, char *out);
 extern afs_int32 rx_SlowWritePacket(struct rx_packet *packet, int offset,
 				    int resid, char *in);
-extern void rxi_freeCBuf(struct rx_packet *c);
 extern int rxi_RoundUpPacket(struct rx_packet *p, unsigned int nb);
 extern int rxi_AllocDataBuf(struct rx_packet *p, int nb, int cla_ss);
 extern void rxi_MorePackets(int apackets);
@@ -447,15 +484,17 @@ extern void rxi_MorePacketsNoLock(int apackets);
 extern void rxi_FreeAllPackets(void);
 extern void rx_CheckPackets(void);
 extern void rxi_FreePacketNoLock(struct rx_packet *p);
-extern int rxi_FreeDataBufsNoLock(struct rx_packet *p, int first);
+extern int rxi_FreeDataBufsNoLock(struct rx_packet *p, afs_uint32 first);
 extern void rxi_RestoreDataBufs(struct rx_packet *p);
 extern int rxi_TrimDataBufs(struct rx_packet *p, int first);
 extern void rxi_FreePacket(struct rx_packet *p);
 extern struct rx_packet *rxi_AllocPacketNoLock(int cla_ss);
 extern struct rx_packet *rxi_AllocPacket(int cla_ss);
+extern int rxi_AllocPackets(int cla_ss, int num_pkts, struct rx_queue *q);
+extern int rxi_FreePackets(int num_pkts, struct rx_queue *q);
 extern struct rx_packet *rxi_AllocSendPacket(register struct rx_call *call,
 					     int want);
-extern int rxi_ReadPacket(int socket, register struct rx_packet *p,
+extern int rxi_ReadPacket(osi_socket socket, register struct rx_packet *p,
 			  afs_uint32 * host, u_short * port);
 extern struct rx_packet *rxi_SplitJumboPacket(register struct rx_packet *p,
 					      afs_int32 host, short port,
@@ -498,14 +537,14 @@ extern int rxi_AdjustDgramPackets(int frags, int mtu);
 /* rx_pthread.c */
 extern void rxi_Delay(int sec);
 extern void rxi_InitializeThreadSupport(void);
-extern void rxi_StartServerProc(void (*proc) (void), int stacksize);
+extern void rxi_StartServerProc(void *(*proc) (void *), int stacksize);
 #ifndef rxi_ReScheduleEvents
 extern void rxi_ReScheduleEvents(void);
 #endif
-extern void rx_ServerProc(void);
+extern void *rx_ServerProc(void *);
 extern void rxi_StartListener(void);
 extern int rxi_Listen(osi_socket sock);
-extern int rxi_Recvmsg(int socket, struct msghdr *msg_p, int flags);
+extern int rxi_Recvmsg(osi_socket socket, struct msghdr *msg_p, int flags);
 extern int rxi_Sendmsg(osi_socket socket, struct msghdr *msg_p, int flags);
 
 
@@ -535,9 +574,6 @@ extern int rx_WritevProc(struct rx_call *call, struct iovec *iov, int nio,
 extern void rxi_FlushWrite(register struct rx_call *call);
 extern void rx_FlushWrite(struct rx_call *call);
 
-/* rx_stream.c */
-
-
 /* rx_trace.c */
 
 
@@ -549,14 +585,14 @@ extern pthread_mutex_t rx_if_mutex;
 extern osi_socket rxi_GetUDPSocket(u_short port);
 extern void osi_AssertFailU(const char *expr, const char *file, int line);
 extern int rx_getAllAddr(afs_int32 * buffer, int maxSize);
-extern void osi_Panic();	/* leave without args till stdarg rewrite */
 extern void rxi_InitPeerParams(struct rx_peer *pp);
-#ifdef  AFS_AIX32_ENV
-#ifndef osi_Alloc
-extern char *osi_Alloc(afs_int32 x);
-extern void osi_Free(char *x, afs_int32 size);
-#endif
-#endif /* AFS_AIX32_ENV */
+extern int rxi_HandleSocketError(int socket);
+
+#if defined(AFS_AIX32_ENV) && !defined(KERNEL)
+extern void *osi_Alloc(afs_int32 x);
+extern void osi_Free(void *x, afs_int32 size);
+#endif /* defined(AFS_AIX32_ENV) && !defined(KERNEL) */
+
 extern void rx_GetIFInfo(void);
 extern void rx_SetNoJumbo(void);
 
