@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/disk.c,v 1.15.4.1 2007/10/30 15:16:47 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ubik/disk.c,v 1.15.4.2 2008/04/02 19:51:56 shadow Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
@@ -798,8 +798,11 @@ udisk_commit(struct ubik_trans *atrans)
 	}
 
 	dbase->version.counter++;	/* bump commit count */
+#if defined(AFS_PTHREAD_ENV) && defined(UBIK_PTHREAD_ENV)
+	assert(pthread_cond_broadcast(&dbase->version_cond) == 0);
+#else
 	LWP_NoYieldSignal(&dbase->version);
-
+#endif
 	code = udisk_LogEnd(dbase, &dbase->version);
 	if (code) {
 	    dbase->version.counter--;
@@ -915,6 +918,10 @@ udisk_end(struct ubik_trans *atrans)
     free(atrans);
 
     /* Wakeup any writers waiting in BeginTrans() */
+#if defined(AFS_PTHREAD_ENV) && defined(UBIK_PTHREAD_ENV)
+	assert(pthread_cond_broadcast(&dbase->flags_cond) == 0);
+#else
     LWP_NoYieldSignal(&dbase->flags);
+#endif
     return 0;
 }
