@@ -19,7 +19,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/uss/uss_vol.c,v 1.10.14.1 2007/02/09 01:00:22 shadow Exp $");
+    ("$Header: /cvs/openafs/src/uss/uss_vol.c,v 1.10.14.3 2007/10/30 15:16:48 shadow Exp $");
 
 #include "uss_vol.h"		/*Interface to this module */
 #include "uss_common.h"		/*Common definitions */
@@ -30,13 +30,7 @@ RCSID
 #include <netdb.h>
 #include <errno.h>
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #include <afs/vlserver.h>
 #include <afs/auth.h>
@@ -739,7 +733,7 @@ uss_vol_CreateVol(a_volname, a_server, a_partition, a_quota, a_mpoint,
     if (!initDone) {
 	code = InitThisModule(NoAuthFlag, uss_ConfDir, uss_Cell);
 	if (code) {
-	    com_err(uss_whoami, code,
+	    afs_com_err(uss_whoami, code,
 		    "while inititializing VLDB connection(s)\n");
 	    return (code);
 	}
