@@ -14,7 +14,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/venus/fstrace.c,v 1.22 2005/10/13 15:12:22 shadow Exp $");
+    ("$Header: /cvs/openafs/src/venus/fstrace.c,v 1.23.2.3 2007/11/26 21:08:45 shadow Exp $");
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -27,13 +27,7 @@ RCSID
 #undef abs
 #include <stdlib.h>
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 
 #include <afs/stds.h>
 #include <afs/cmd.h>
@@ -87,6 +81,7 @@ struct logInfo {
 } *allInfo = 0;
 
 char dumpFileName[256] = "";
+void
 RegisterIclDumpFileName(name)
      char *name;
 {
@@ -258,7 +253,7 @@ CheckTypes(bufferp, typesp, typeCount)
 #if defined(AFS_SGI61_ENV) && !defined(AFS_SGI62_ENV)
 #define uint64_t long long
 #endif
-static
+static void
 DisplayRecord(outFilep, alp, rsize)
      FILE *outFilep;
      register afs_int32 *alp;
@@ -285,6 +280,7 @@ DisplayRecord(outFilep, alp, rsize)
     int status;
     int printed;		/* did we print the string yet? */
     afs_int32 *tlp;
+    time_t tmv;
 
     /* decode parameters */
     temp = alp[0];		/* type encoded in low-order 24 bits, t0 high */
@@ -354,7 +350,8 @@ DisplayRecord(outFilep, alp, rsize)
 #endif /* AFS_SGI64_ENV */
 	    break;
 	case ICL_TYPE_UNIXDATE:
-	    printfParms[pfpix++] = (long)ctime((time_t *) & alp[pix]);
+	    tmv = alp[pix];
+	    printfParms[pfpix++] = (long)ctime(&tmv);
 	    break;
 	default:
 	    printf("DisplayRecord: Bad type %d in decode switch.\n", type);
@@ -450,8 +447,9 @@ DisplayRecord(outFilep, alp, rsize)
     }
     if (!printed) {
 	if (alp[1] == ICL_INFO_TIMESTAMP) {
+	    tmv = alp[4];
 	    fprintf(outFilep, "time %d.%06d, pid %u: %s\n", alp[3] / 1000000,
-		    alp[3] % 1000000, alp[2], ctime((time_t *) & alp[4]));
+		    alp[3] % 1000000, alp[2], ctime(&tmv));
 	} else {
 	    fprintf(outFilep, "raw op %d, time %d.%06d, pid %u\n", alp[1],
 		    alp[3] / 1000000, alp[3] % 1000000, alp[2]);
@@ -500,8 +498,9 @@ DisplayRecord(outFilep, alp, rsize)
 		    fprintf(outFilep, "p%d:%s ", i, (char *)&alp[pix]);
 		    break;
 		case ICL_TYPE_UNIXDATE:
+		    tmv = alp[pix];
 		    fprintf(outFilep, "p%d:%s ", i,
-			    ctime((time_t *) & alp[pix]));
+			    ctime(&tmv));
 		    break;
 		default:
 		    printf
@@ -2709,10 +2708,8 @@ GetBulkLogInfo()
 }
 
 
-static
-DoDump(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoDump(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code = 0;
     afs_int32 tcode;
@@ -2788,7 +2785,7 @@ SetUpDump()
     struct cmd_syndesc *dumpSyntax;
 
     dumpSyntax =
-	cmd_CreateSyntax("dump", DoDump, (char *)NULL, "dump AFS trace logs");
+	cmd_CreateSyntax("dump", DoDump, NULL, "dump AFS trace logs");
     (void)cmd_AddParm(dumpSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
     (void)cmd_AddParm(dumpSyntax, "-follow", CMD_SINGLE, CMD_OPTIONAL,
 		      "log_name");
@@ -2798,10 +2795,8 @@ SetUpDump()
 		      "seconds_between_reads");
 }
 
-static
-DoShowLog(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoShowLog(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2864,17 +2859,15 @@ SetUpShowLog()
     struct cmd_syndesc *showSyntax;
 
     showSyntax =
-	cmd_CreateSyntax("lslog", DoShowLog, (char *)NULL,
+	cmd_CreateSyntax("lslog", DoShowLog, NULL,
 			 "list available logs");
     (void)cmd_AddParm(showSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
     (void)cmd_AddParm(showSyntax, "-log", CMD_LIST, CMD_OPTIONAL, "log_name");
     (void)cmd_AddParm(showSyntax, "-long", CMD_FLAG, CMD_OPTIONAL, "");
 }
 
-static
-DoShowSet(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoShowSet(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2922,15 +2915,13 @@ SetUpShowSet()
     struct cmd_syndesc *showSyntax;
 
     showSyntax =
-	cmd_CreateSyntax("lsset", DoShowSet, (char *)NULL,
+	cmd_CreateSyntax("lsset", DoShowSet, NULL,
 			 "list available event sets");
     (void)cmd_AddParm(showSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
 }
 
-static
-DoClear(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoClear(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -2981,7 +2972,7 @@ SetUpClear()
     struct cmd_syndesc *clearSyntax;
 
     clearSyntax =
-	cmd_CreateSyntax("clear", DoClear, (char *)NULL,
+	cmd_CreateSyntax("clear", DoClear, NULL,
 			 "clear logs by logname or by event set");
     (void)cmd_AddParm(clearSyntax, "-set", CMD_LIST, CMD_OPTIONAL,
 		      "set_name");
@@ -2989,10 +2980,8 @@ SetUpClear()
 		      "log_name");
 }
 
-static
-DoSet(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoSet(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -3070,7 +3059,7 @@ SetUpSet()
     struct cmd_syndesc *setSyntax;
 
     setSyntax =
-	cmd_CreateSyntax("setset", DoSet, (char *)NULL,
+	cmd_CreateSyntax("setset", DoSet, NULL,
 			 "set state of event sets");
     (void)cmd_AddParm(setSyntax, "-set", CMD_LIST, CMD_OPTIONAL, "set_name");
     (void)cmd_AddParm(setSyntax, "-active", CMD_FLAG, CMD_OPTIONAL, "");
@@ -3078,10 +3067,8 @@ SetUpSet()
     (void)cmd_AddParm(setSyntax, "-dormant", CMD_FLAG, CMD_OPTIONAL, "");
 }
 
-static
-DoResize(as, arock)
-     register struct cmd_syndesc *as;
-     char *arock;
+static int
+DoResize(register struct cmd_syndesc *as, void *arock)
 {
     afs_int32 retVal = 0;
     afs_int32 code = 0;
@@ -3129,7 +3116,7 @@ SetUpResize()
     struct cmd_syndesc *setsizeSyntax;
 
     setsizeSyntax =
-	cmd_CreateSyntax("setlog", DoResize, (char *)NULL,
+	cmd_CreateSyntax("setlog", DoResize, NULL,
 			 "set the size of a log");
     (void)cmd_AddParm(setsizeSyntax, "-log", CMD_LIST, CMD_OPTIONAL,
 		      "log_name");
